/***********************************************************************
SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1

Copyright 2015 John C. P. Hemmings 

An adaptation of code from the 2013 MarMOT 1.1 release (MarMOT 1.1.0) 
 - Copyright 2013 Natural Environment Research Council, UK  

Author: John Hemmings
e-mail: marmot@wessexenv.mail1.co.uk 

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

***********************************************************************/
/***********************************************************************

 gamicro.c - John Hemmings - 14 Jan 2015 

 Optimizer code based on a micro-genetic algorithm (Krishnakumar, 1989)
 incorporating elitism, deterministic tournament selection and 
 single-point crossover.  Uniform crossover, recommended by Carroll
 (1996), is provided as an alternative. The method implemented here is 
 designed to be functionally equivalent to that presented in Carroll's 
 FORTRAN Genetic Algorithm Driver (Carroll, 2001) but uses a native 
 bit-field representation of the population gene pool. Because of this, 
 fewer random variates are used during genome creation implying that
 results should not be expected to be identical to those obtained using 
 Carroll's driver with the same random number generator and initial seed.
 Note that one or two children per parent pair are supported, whereas
 Carroll's driver always uses the one child option for the micro GA.

 References:

 Krishnakumar, K., "Micro-Genetic Algorithms for Stationary and
 Non-Stationary Function Optimization," SPIE: Intelligent Control and
 Adaptive Systems, Vol. 1196, Philadelphia, PA, 1989. 
 doi:10.1117/12.969927.

 Carroll, D. L., "Genetic Algorithms and Optimizing Chemical 
 Oxygen-Iodine Lasers," Developments in Theoretical and Applied 
 Mechanics, Vol. XVIII, eds. H. B. Wilson, R. C. Batra, C. W. Bert, 
 A. M. J.  Davis, R. A. Schapery, D. S. Stewart, and F. F. Swinson, 
 School of Engineering, The University of Alabama, 1996, pp.411-424.

 Carroll, D. L., FORTRAN Genetic Algorithm Driver v1.7.0, 2001 
 (available for non-commercial use from 
 http://www.cuaerospace.com/Technology/GeneticAlgorithm).

***********************************************************************/

#include "optimizer.h"
#include "genome.h"
#include "garandom.h"

#ifdef echo_random
  #define RAN_ECHO 1   /* Echo random variates to stdout */
#else
  #define RAN_ECHO 0   
#endif


struct gaworkspacerec {

  Population parent;                  /* Population gene pool */
  Population child;                   /* Child gene pool for next generation */
  Genome popbest;                     /* Genome of fittest individual */
  double fitnessbest;                 /* Max. fitness (- function value) */
  Genome crossbits;                   /* Crossover spec. as a bit mask */
  int popconverged;                   /* Flag for convergence by micro-GA
                                         criterion */
  Parmvec_Int uparmvec;               /* Integer representation of parameter 
                                         vector for an individual */
  char *genome_fieldlist;             /* Buffer for population gene pool in
                                         string format */
  char **genometxt;                   /* Population pool in string format */
};

static void ga_init (struct gadatarec *, struct gaworkspacerec *,
                     OBJF_FLOAT_TYPE *);
static void ga_generation (struct gadatarec *, struct gaworkspacerec *,
                           int , double [], int *);
static int ga_writegenreport (struct gadatarec *, struct gaworkspacerec *,
                              OBJF_FLOAT_TYPE *, double [], int *);
static Genome bestoftwo (int, Population, Genome, double [], int *);
static void ga_eval (struct gadatarec *, struct gaworkspacerec *,
                     OBJF_FLOAT_TYPE *, double [], int *);


int minimize_ga (struct gadatarec *gi, OBJF_FLOAT_TYPE *parmval, 
                 double fitness[], int *psetbest)

{ struct gaworkspacerec gwk;
  struct gactrlrec *ctrl;
  struct parmctrlrec *parmctrl;
  int nbitsg;
  int status; 
  int i;
  char *txt;

  ctrl = gi->ctrl;
  parmctrl = gi->parmctrl;

  /* Generate mapping information for coding/decoding parameter vector 
     and other genome handling functions provided by genome.c */

  nbitsg = make_codemap(parmctrl->nparm, parmctrl->nbits);
  if (nbitsg == 0) return MEMERR;

  /* Allocate workspace */

  gwk.parent = malloc_population(ctrl->npopsiz);
  if (! gwk.parent) return MEMERR;

  gwk.child = malloc_population(ctrl->npopsiz);
  if (! gwk.child) return MEMERR;

  gwk.popbest = malloc_genome();
  if (! gwk.popbest) return MEMERR;
  
  gwk.crossbits = malloc_genome();
  if (! gwk.crossbits) return MEMERR;

  gwk.uparmvec = malloc_uparmvec();
  if (! gwk.uparmvec) return MEMERR;

  gwk.genome_fieldlist = 
    (char *) malloc(ctrl->npopsiz * (nbitsg + 1));
  if (! gwk.genome_fieldlist) return MEMERR;

  gwk.genometxt = (char **) malloc(ctrl->npopsiz * sizeof(char *));
  if (! gwk.genometxt) return MEMERR;

  /* Initialize random number generator */

  if (gi->logfile)
    { txt = randomdescrip(&status);
      if (txt) 
        { fprintf(gi->logfile,
            "\nSource of Random Variates for Genetic Algorithm:\n%s\n", txt);
          fflush(gi->logfile);
        }
    }
 
  randominit(ctrl->seed);

  /* Initialize population */

  ga_init(gi,&gwk,parmval);

  /* Run generations */

  gi->gennum = 1;
  gwk.popconverged = 0;
  for (i = 0; i < ctrl->maxgen && !gwk.popconverged; i++)
    { if (i == 0) *psetbest = 0;
      else ga_generation(gi,&gwk,nbitsg,fitness,psetbest);
      ga_eval(gi,&gwk,parmval,fitness,psetbest); 
      if (gi->logfile)
	{ status = ga_writegenreport(gi,&gwk,parmval,fitness,psetbest);
	  if (status != OK) return status; 
	}
    }

  /* Free workspace */

  clear_codemap();  
  free_population(gwk.parent);
  free_population(gwk.child);
  free_genome(gwk.popbest);
  free_genome(gwk.crossbits);
  free_uparmvec(gwk.uparmvec);
  free(gwk.genome_fieldlist);
  free(gwk.genometxt);

  return OK; 
}


static void ga_init (struct gadatarec *gi, struct gaworkspacerec *gwk,
                     OBJF_FLOAT_TYPE *parmval)

{ struct gactrlrec *ctrl;
  struct parmctrlrec *parmctrl;
  OBJF_FLOAT_TYPE *parmvec;
  double x;
  int i,j;

  ctrl = gi->ctrl;
  parmctrl = gi->parmctrl;

  if (ctrl->psetini == 1)

    { for (i = 0; i < ctrl->npopsiz; i++)

        { /* Get parameter values for individual 'i' */

          parmvec = &parmval[i * parmctrl->nparm];

          /* Convert to integer representation */

          for (j = 0; j < parmctrl->nparm; j++)

            { if (parmctrl->logtrans[j])
                x = (log10(parmvec[j]) - parmctrl->logmin[j]) /
                    parmctrl->step[j];
              else
                x = (parmvec[j] - parmctrl->min[j]) /
                    parmctrl->step[j];

              if (x < 0) gwk->uparmvec[j] = 0;
              else if (x > genome_uparmmax) gwk->uparmvec[j] = genome_uparmmax;
              else gwk->uparmvec[j] = x;
            }

          genomecode(gwk->uparmvec,gwk->parent[i]);
        }
    }

  else
    for (i = 0; i < ctrl->npopsiz; i++) genomecreate(RAN_ECHO,gwk->parent[i]);
}


static void ga_generation (struct gadatarec *gi, struct gaworkspacerec *gwk, 
                           int nbitsg, double fitness[], int *psetbest)

{ FILE *outfile;
  struct gactrlrec *ctrl;
  Genome mate1,mate2;  
  int ncrosspop,ncross;
  int i,inext,irand;
  int elitefound;
  int count;
  double dfrac;

  ctrl = gi->ctrl;
  outfile = gi->logfile;

  /* Do tournament selection and crossover */

  ncrosspop = 0;
  inext = ctrl->npopsiz-1;

  for (i = 0; i < ctrl->npopsiz; i += ctrl->nchild)

    { /* Select mates for breeding */

      mate1 = bestoftwo(ctrl->npopsiz,gwk->parent,gwk->child[0],fitness,&inext);
      mate2 = bestoftwo(ctrl->npopsiz,gwk->parent,gwk->child[0],fitness,&inext);

      /* Set up bitmap for crossover */

      if (ctrl->iunifrm) 
        crossover_uniform(RAN_ECHO,ctrl->pcross,gwk->crossbits,&ncross);
      else 
        crossover_point(RAN_ECHO,ctrl->pcross,gwk->crossbits,&ncross);

      /* Apply crossover to get child/children */

      genomecrossover(gwk->crossbits,mate1,mate2,gwk->child[i]);
      if (ctrl->nchild == 2) 
        genomecrossover(gwk->crossbits,mate2,mate1,gwk->child[i+1]);

      ncrosspop += ncross;
    }

  if (outfile) fprintf(outfile,"Number of crossovers = %d\n",ncrosspop);

  /* Write children back into parent array for new generation and
     check to see if the best parent was replicated (record new position) */

  *psetbest = 0;
  elitefound = 0;
  for (i = 0; i < ctrl->npopsiz; i++)
    { genomecopy(gwk->child[i],gwk->parent[i]); 
      if (!elitefound && genomematch(gwk->parent[i],gwk->popbest))
        { *psetbest = i + 1;
          elitefound = 1;
        }
    }

  /* If best parent was not replicated then reproduce best parent into a 
     random slot (record new position) */

  if (!elitefound)

    { irand = ga_random_int(0,ctrl->npopsiz - 1,RAN_ECHO);
      genomecopy(gwk->popbest,gwk->parent[irand]);
      *psetbest = irand + 1;

      if (outfile)
        fprintf(outfile,"Elitist reproduction to new individual %d\n",
                irand + 1);
    }
 
  /* New generation: count number of bits different from best parent */

  gi->gennum++;

  count = 0;
  for (i = 0; i < ctrl->npopsiz; i++)
    if (i != *psetbest - 1) count += genomediff(gwk->parent[i],gwk->popbest);

  /* If less than 5% of bits differ then consider population to be converged
      - if more generations needed, restart with best parent and random others 
        (record new position of best parent) */

  dfrac = (double)count / ((ctrl->npopsiz - 1) * nbitsg);

  if (outfile) 
    fprintf(outfile,
      "New generation: %.3g%% of bits in rest of gene pool differ from elite individual\n",
            dfrac*100);

  if (dfrac < 0.05)

    { gwk->popconverged = 1;
      if (outfile) fprintf(outfile,"Population converged\n");

      if (gi->gennum < ctrl->mingen)

        { genomecopy(gwk->popbest,gwk->parent[0]);
          *psetbest = 1;

          for (i = 1; i < ctrl->npopsiz; i++) 
            genomecreate(RAN_ECHO,gwk->parent[i]);

          if (outfile) 
            fprintf(outfile,
 "%%%%%%%%%%%%%%  Restart micro-population at generation %d  %%%%%%%%%%%%%%\n",
                    gi->gennum);

          gwk->popconverged = 0;
        }
    }
}


static Genome bestoftwo (int npopsiz, Population parent, Genome gtmp, 
                         double fitness[], int *inext)

{ double x;
  int i,j;

  /* If all available pairs used, shuffle population before continuing */

  if (*inext + 1 >= npopsiz)

    { for (i = 0; i < npopsiz - 1; i++)

        { j = ga_random_int(i + 1, npopsiz - 1, RAN_ECHO); 

          genomecopy(parent[j],gtmp);
          genomecopy(parent[i],parent[j]);
          genomecopy(gtmp,parent[i]);
           
          x = fitness[j];
          fitness[j]=fitness[i];
          fitness[i]= x;
        }

      *inext = 0;
    }

  /* Pick best individual of next pair */ 

  i = *inext;
  j = *inext + 1;

  (*inext) += 2;

  if (fitness[i] > fitness[j]) return parent[i];
  else return parent[j];
}


static void ga_eval (struct gadatarec *gi, struct gaworkspacerec *gwk,
                     OBJF_FLOAT_TYPE *parmval, double fitness[], int *psetbest) 

{ struct parmctrlrec *parmctrl;
  OBJF_FLOAT_TYPE *parmvec,*parmvec_prev;
  double func,best;
  int i,j,ibest,duplicate;

  parmctrl = gi->parmctrl;

  for (i = 0; i < gi->ctrl->npopsiz; i++) 

    { /* Check whether individual 'i' is identical to the previous one */
 
      if (i == 0) duplicate = 0; 
      else duplicate = genomematch(gwk->parent[i],gwk->parent[i - 1]);

      /* Get parameter values for individual 'i' */ 

      parmvec = &parmval[i * parmctrl->nparm];

      if (duplicate)

        { parmvec_prev = &parmval[(i - 1) * parmctrl->nparm];
          for (j = 0; j < parmctrl->nparm; j++) parmvec[j]=parmvec_prev[j];
        }

      else

        { genomedecode(gwk->parent[i],gwk->uparmvec);

          for (j = 0; j < parmctrl->nparm; j++)   

            { if (parmctrl->logtrans[j])
                parmvec[j] = pow(10, parmctrl->logmin[j] +
                                 parmctrl->step[j] * (0.5 + gwk->uparmvec[j]));
              else
                parmvec[j] = parmctrl->min[j] + 
                             parmctrl->step[j] * (0.5 + gwk->uparmvec[j]);
            }
        } 

      /* Get fitness if already known and call function driver. 
         Driver evaluates objective function if needed or returns 
         the known fitness. */
      
      gi->psetnum = i + 1;   
      if (duplicate) gi->knownfitness = &fitness[i - 1];  
      else if (gi->psetnum == *psetbest) gi->knownfitness = &gwk->fitnessbest;
      else gi->knownfitness = NULL;

      fitness[i] = (*gi->func)(gi,parmvec);

      /* Check whether fitness of individual 'i' is best */

      if (i == 0 || fitness[i] > best)
        { best = fitness[i];
          ibest = i; 
        }
    }

  /* Update record of best individual */

  *psetbest = ibest + 1; 
  genomecopy(gwk->parent[ibest], gwk->popbest);
  gwk->fitnessbest = best;
}


static int ga_writegenreport (struct gadatarec *gi, struct gaworkspacerec *gwk,
                              OBJF_FLOAT_TYPE *parmval, double fitness[],
                              int *psetbest)

{ FILE *outfile;
  struct parmctrlrec *parmctrl;
  char *varname[N_FREEPARM_MAX+2];
  int sigdig[NFIELD_MAX];
  int fw[N_FREEPARM_MAX+2];
  int status,nvar,npage,pageno,i,j,k,ipage;
  OBJF_FLOAT_TYPE sum,pval,pmin,pmax;
  time_t systime;
  char *s;
  int npopsiz,nparm;

  outfile = gi->logfile;
  if (!outfile) return OK;

  npopsiz = gi->ctrl->npopsiz;
  parmctrl = gi->parmctrl;
  nparm = parmctrl->nparm;

  /* Convert genes to string format */

  s = gwk->genome_fieldlist;
  for (i = 0; i < npopsiz; i++) 
    { gwk->genometxt[i] = s; 
      s += genomestr(gwk->parent[i], s);
    }

  fprintf(outfile, "\n#################  Generation %d  #################\n\n",
          gi->gennum);

  time(&systime);
  s = ctime(&systime);
  fprintf(outfile,"%s\n",s);

  /* Set variable names
     Remove string fields in page buffer for numerical data to force
     use of values in number field */

  nvar = 0;
  varname[nvar] = "individual";
  for (i = 0; i < NPAGEREC_MAX; i++) dpage[i].field[nvar] = NULL;
  nvar++;
  varname[nvar] = "genome";
  nvar++;
  for (j = 0; j < nparm; j++)
    { varname[nvar] = gi->parmname[j];
      for (i = 0; i < NPAGEREC_MAX; i++) dpage[i].field[nvar] = NULL;
      nvar++;
    }
  varname[nvar] = "func";
  for (i = 0; i < NPAGEREC_MAX; i++) dpage[i].field[nvar] = NULL;
  nvar++;

  npage = 1 + (int)((npopsiz - 1) / NPAGEREC_MAX);
  pageno = 1;
  i = 0;
  while (i < npopsiz)

    { /* Set up table data on current logical page */

      for (ipage = 0; ipage < NPAGEREC_MAX && i < npopsiz; ipage++)

        { k = 0;
          dpage[ipage].x[k++] = (double)(i + 1);
          dpage[ipage].field[k++] = gwk->genometxt[i];
          for (j = 0; j < nparm; j++) 
            dpage[ipage].x[k++] = (double)parmval[i * nparm + j];
          dpage[ipage].x[k] = (double)(-fitness[i]);

          i++;
        }

      /* Write logical page */

      sigdig[0] = DEFAULT;
      k = 2;
      for (j = 0; j < nparm; j++) sigdig[k++] = OBJF_SIGDIG_DEF;
      sigdig[k] = OBJF_SIGDIG_DEF;

      setcolwidths(nvar,varname,ipage,dpage,sigdig,CWIDTH_MAX,fw);

      if (i > NPAGEREC_MAX) fprintf(outfile,"\n");
      if (npage > 1) fprintf(outfile,"[Page %d of %d]\n",pageno++,npage);
      status=writelogicalpage(outfile,nvar,1,varname,fw,sigdig,DEFAULT,
                              COLSEP_LOG,PAGEWIDTH,ipage,dpage);
      if (status != OK) return status;
    }

  fprintf(outfile, "\n");

  fprintf(outfile, "----- Summary Statistics -----\n\n");

  /* Set variable names
     Remove string fields in page buffer for numerical data to force
     use of values in number field */

  nvar = 0;
  varname[nvar] = "parmname";
  nvar++;
  varname[nvar] = "mean";
  for (i = 0; i < NPAGEREC_MAX; i++) dpage[i].field[nvar] = NULL;
  nvar++;
  varname[nvar] = "range";
  for (i = 0; i < NPAGEREC_MAX; i++) dpage[i].field[nvar] = NULL;
  nvar++;
  varname[nvar] = "min";
  for (i = 0; i < NPAGEREC_MAX; i++) dpage[i].field[nvar] = NULL;
  nvar++;
  varname[nvar]="max";
  for (i = 0; i < NPAGEREC_MAX; i++) dpage[i].field[nvar] = NULL;
  nvar++;

  npage = 1 + (int)((nparm - 1) / NPAGEREC_MAX);
  pageno = 1;
  i = 0;
  while (i < nparm)

    { /* Set up table data on current logical page */

     for (ipage = 0; ipage < NPAGEREC_MAX && i < nparm; ipage++)

        { sum = 0;
          for (j = 0; j < npopsiz; j++)
            { pval = parmval[j * nparm + i];
              if (j == 0 || pval < pmin) pmin = pval;
              if (j == 0 || pval > pmax) pmax = pval;
              sum += pval;
            }

          k = 0;
          dpage[ipage].field[k++] = gi->parmname[i];
          dpage[ipage].x[k++] = (double)sum / npopsiz;
          dpage[ipage].x[k++] = (double)(pmax - pmin);
          dpage[ipage].x[k++] = (double)pmin;
          dpage[ipage].x[k++] = (double)pmax;

          i++;
        }

      /* Write logical page */

      sigdig[1] = OBJF_SIGDIG_DEF;
      sigdig[2] = OBJF_SIGDIG_DEF;
      sigdig[3] = OBJF_SIGDIG_DEF;
      sigdig[4] = OBJF_SIGDIG_DEF;

      setcolwidths(nvar, varname, ipage, dpage, sigdig, CWIDTH_MAX, fw);

      if (i > NPAGEREC_MAX) fprintf(outfile,"\n");
      if (npage > 1) fprintf(outfile,"[Page %d of %d]\n",pageno++,npage);
      status = writelogicalpage(outfile,nvar,1,varname,fw,sigdig,DEFAULT,
                                COLSEP_LOG,PAGEWIDTH,ipage,dpage);
      if (status != OK) return status;
    }

  fprintf(outfile, "\n");

  sum = 0;
  for (j = 0; j < npopsiz; j++) sum += -fitness[j];
  fprintf(outfile,"Mean function value     = %.*g\n",
          OBJF_SIGDIG_DEF,sum / npopsiz);
  fprintf(outfile,"Minimum function value  = %.*g (individual = %d)\n",
          OBJF_SIGDIG_DEF,-fitness[*psetbest - 1],*psetbest);

  fprintf(outfile, 
          "\n####################################################\n\n");
  fflush(outfile);

  return OK;
}

