/***********************************************************************
SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1
***********************************************************************/
/***********************************************************************

SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013

Copyright Natural Environment Research Council, UK

Author: John Hemmings, National Oceanography Centre 
e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013) 
Alternative e-mail for author: marmot@wessexenv.mail1.co.uk 

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

***********************************************************************/
/* powell.c - John Hemmings (31 Oct 2012) */ 

/* Contains an implementation of the Powell (1964) direction set algorithm for
   multi-dimensional function minimization without using gradients.
   This implementation uses the Brent (1973) algorithm for 1-D minimization 
   along successive directions in parameter space. A further algorithm is
   employed to bracket a minimum for initializing 1-D minimization. It is a 
   essentially a re-working of that suggested by Press et al (1992). 

   References:
   Powell, M.J.D. 1964. An efficient method for ﬁnding the minimum of a 
   function of several variables without calculating derivatives. 
   Computer J. 7, 155-162. 
   Brent, R.P. 1973. Algorithms for Minimization without Derivatives. 
   Prentice Hall, Englewood Cliﬀs, N.J., 61-80.
   Press, W. H., B. P. Flannery, S. A. Teukolsky and W. T. Vetterling. 1992. 
   Numerical Recipes in C: the Art of Scientiﬁc Computing. Cambridge Univ. 
   Press, Cambridge. 
*/ 

#include "optimizer.h"  

#define CGOLD 0.3819660 /* Fraction for "golden section" search */
#define GOLD 1.618034   /* Default magnification ratio for successive steps 
                           to bracket minimum = (1-CGOLD)/CGOLD */ 
#define GLIMIT 100.0    /* Maximum magnification for a parabolic step */

static int linemin (struct powelldatarec *, double [], double [], double *);
static int bracket_minimum (struct powelldatarec *, double [], double [], 
                            double, double *, double *, double *, double *, 
                            double *, double *);
static int minimize_brent (struct powelldatarec *, double [], double [], double,
                           double, double, double, double, double *, double *);
static void swap_points (double *, double *, double *, double *);
static double funcval (struct powelldatarec *, double [], double [], double);

/* minimize_powell: use the direction set method of Powell (1964) to locate 
   a function minimum 'fmin' in parameter space starting at 'p'. Optimizer data
   defining the specifics of the problem and available workspace are provided 
   in 'optdata'. One iteration involves a sequence of minimizations along a 
   set of 'n' directions, where 'n' is the dimensionality of the parameter
   space. A potential new direction is given by the resulting displacement and 
   the function is evaluated at a trial point further on in that direction. 
   If the new direction looks promising, we minimize along it and it then 
   replaces the direction of largest function change in the direction set.
   (The discarded direction is likely to be a large component of the new 
   direction.) Iteration continues until the difference in the function value 
   between successive iterations is less than a prescribed fractional tolerance
   'ffrctol' or the maximum number of allowed iterations is reached. 
   Returns 0 on a successful convergence or 1 otherwise. */ 

int minimize_powell (struct powelldatarec *optdata, double p[], double *fmin,
                     char **errtxt) 

{ double **direction;          /* direction set */
  double *dir;                 /* new direction vector */ 
  int n,i,ideltamax,j,status;
  double f,deltamax;
  double *p0,*pext;
  double fp0,fpext;
  double d1,d2,s;
  FILE *logfile;
  struct powellctrlrec *ctrl;

  *errtxt=NULL;
  logfile=optdata->logfile;
  ctrl=optdata->ctrl;

  /* Initialize direction set: use unit length basis vectors
     These will be re-scaled to appropriate lengths by 'linemin' */ 

  n=optdata->n;

  direction=optdata->direction;
  for (i=0;i<n;i++)
    for (j=0;j<n;j++) direction[i][j]=(i==j?1:0.0); 

  p0=optdata->p0;
  pext=optdata->pext;
  dir=optdata->dir;

  optdata->iternum=0;
  optdata->dirnum=0;
  optdata->evalnum=0;

  *fmin=(*optdata->func)(optdata,p);

  while (1)

    { if (++optdata->iternum > ctrl->maxiter)
        { *errtxt="Powell algorithm exceeded maximum iterations";
          if (logfile) 
            { fprintf(logfile,"\n%s\n",*errtxt); 
              fflush(logfile);
            }
          return 1;
        }

      if (logfile)
        { fprintf(logfile,"\nPOWELL ITERATION %d\n",optdata->iternum);
          for (i=0;i<n;i++)
            { fprintf(logfile,"standardized direction %d: ",i+1);
              s=0.0;
              for (j=0;j<n;j++) s+=direction[i][j]*direction[i][j];
              s=1.0/sqrt(s);
              for (j=0; direction[i][j]==0.0 && j<n; j++);
              if (direction[i][j]<0.0) s=-s;
              for (j=0;j<n;j++) fprintf(logfile,"%g ",direction[i][j]*s); 
              fprintf(logfile,"\n");
            }
          for (i=0;i<n;i++)
            { fprintf(logfile,"direction %d: ",i+1);
              for (j=0;j<n;j++) fprintf(logfile,"%g ",direction[i][j]);
              fprintf(logfile,"\n");
            }
          fflush(logfile);
        }

      /* Step 1: Mimimize along current set of directions 
         recording largest function change in a single direction and the
         index for that direction */

      for (j=0;j<n;j++) p0[j]=p[j];
      fp0=(*fmin); 

      ideltamax=0;
      deltamax=0.0;

      for (i=0;i<n;i++) 

        { optdata->dirnum=i+1;  
          optdata->evalnum=0;

          f=(*fmin);
          status=linemin(optdata,p,direction[i],fmin);
          if (status!=0) 
            { *errtxt="line minimization failed";
              return 1; 
            }

         if (fabs(f-(*fmin)) > deltamax) 
           { deltamax=fabs(f-(*fmin));
             ideltamax=i;
           }
        }

      /* Return success for a 1-dimensional minimization */

      if (n==1) return 0;

      /* Step 2: Evaluate function at a new point co-linear with initial and 
         final points in Step 1 to get three equidistant points (p0,p,pext) */

      optdata->dirnum++; 
      optdata->evalnum=0;
      for (j=0;j<n;j++) 
        { pext[j]=2.0*p[j]-p0[j];
          dir[j]=p[j]-p0[j];
        }
      fpext=(*optdata->func)(optdata,pext);

      /* Step 3: If average direction from Step 1 still looks promising then 
         minimize along it and add it to direction set discarding direction
         of largest function change (new direction is scaled in 'linemin') */ 

      if (f < fp0) 

        { d1=fp0-(*fmin)-deltamax;
          d2=fp0-fpext; 

          if ((fp0-2.0*(*fmin)+fpext)*d1*d1 < 0.5*deltamax*d2*d2)

            { if (logfile)
                { fprintf(logfile,
                    "\nNew direction to replace direction %d\n",ideltamax+1);
                  fflush(logfile);
                }

              status=linemin(optdata,p,dir,fmin);
              if (status!=0) 
                { *errtxt="line minimization failed";
                  return 1; 
                }

              for (i=ideltamax;i<n-1;i++)
                for (j=0;j<n;j++) direction[i][j]=direction[i+1][j]; 
              for (j=0;j<n;j++) direction[n-1][j]=dir[j];
            }
        }

      if (logfile)
        { fprintf(logfile,"\nPowell iteration %d completed: f(",
                  optdata->iternum);
          for (j=0;j<n-1;j++) fprintf(logfile,"%g,",p[j]); 
          fprintf(logfile,"%g)=%g\n",p[n-1],*fmin);
          fflush(logfile);
        }

      /* Check for convergence */

      if (2.0*fabs(fp0-(*fmin)) <= ctrl->ffrctol*(fabs(fp0)+fabs(*fmin))) 
        { if (logfile)
            { fprintf(logfile,"Powell algorithm converged\n");
              fflush(logfile);
            }
          return 0;
        }
    }
}


/* linemin: perform minimization on a line through point 'p' parallel to the 
   vector 'line'. The minimum function value is returned in 'fmin' and 'p' is 
   set to the location of the minimum. 'line' is re-scaled to match the 
   distance and direction moved giving an appropriate trial step size for 
   starting subsequent minimizations in the same direction. */

static int linemin (struct powelldatarec *optdata, double p[],
                    double line[], double *fmin)

{ int n,i,status; 
  double xleft,xright,xmin;
  double fleft,fright;
  double scaling;
  FILE *logfile;

  n=optdata->n;
  logfile=optdata->logfile;

  /* Get scaling factor for adjusting tolerance */

  scaling=0.0;
  for (i=0;i<n;i++) scaling+=line[i]*line[i];
  scaling=1.0/sqrt(scaling);


  if (logfile)
    { fprintf(logfile,"\nLINE MINIMIZATION ORIGIN (x=0): ");
      for (i=0;i<n;i++) fprintf(logfile,"%g ",p[i]);
      fprintf(logfile,"\n");
      fprintf(logfile,"LINE MINIMIZATION DIRECTION %d: ",optdata->dirnum);
      for (i=0;i<n;i++) fprintf(logfile,"%g ",line[i]);
      fprintf(logfile,"\nLINE MINIMIZATION UNIT (x=1): %g",1.0/scaling);
      fprintf(logfile,"\nLine minimization x,f(x): 0 %.15g",*fmin);
      fprintf(logfile,"\n");
      fflush(logfile);
    }

  /* Find an interval in direction 'line' from point 'p' that is known to 
     include a minimum and locate using Brent's algorithm 
     (x unit is the length of the direction vector) */

  status=bracket_minimum(optdata,p,line,scaling,&xleft,&xright,&xmin,
                         &fleft,&fright,fmin);
  if (status==0)
    status=minimize_brent(optdata,p,line,scaling,xleft,xright,fleft,fright,
                          &xmin,fmin);

  /* Re-scale direction vector and update location */

  if (xmin!=0.0) for (i=0;i<n;i++) 
    { line[i]*=xmin;
      p[i]+=line[i]; 
    }

  return status;
}


/* bracket_minimum: attempt to initialize a line minimization from 'origin' 
   along the direction 'line' by bracketing a minimum. On entry, 'fx' is taken 
   to be the function value at 'origin' (where we define x=0).
   On exit, 'x' is the lowest point found. If successful, 'xleft' and 'xright' 
   give the bracket points. Otherwise they indicate the last segment searched. 
   The function values are returned for all three points. 
   The tolerance settings for the Brent algorithm in 'optdata' are used to
   avoid evaluating the function at points that are too close together.
   The tolerance is adjusted if it is less than the global tolerance 'xabstol' 
   scaled to 'x' units. ('x' is a multiple of the length of vector 'line'.)
   The search is stopped if the curve appears to be flat when sampled at 3 
   consective points (ignoring backward steps) and this condition persists for
   more than 'maxiterbracket0' iterations. The maximum number of 
   iterations is defined by 'maxiterbracket'. This includes any additional
   function evaluations that may be needed to find an initial slope if the
   first two trial points are level. The number of such function 
   evaluations is limited to 'maxiterbracket0'. */

static int bracket_minimum (struct powelldatarec *optdata, 
                            double origin[], double line[], double scaling,
                            double *xleft, double *xright, double *x, 
                            double *fleft, double *fright, double *fx)

{ double a,b,c,u,fa,fb,fc,fu,xtol;
  double r,q,ulim,tol,tolmin; 
  int iternum,stopcount,stop,parabolic,status,prx;
  FILE *logfile;
  struct powellctrlrec *ctrl;

  logfile=optdata->logfile;
  ctrl=optdata->ctrl;

  /* Get a pair of points a,b to start (tolerance is calculated as in 
     minimize_brent and used to prevent the trial points being too close) */ 

  tol=ctrl->xabstolbrent;
  tolmin=ctrl->xabstol*scaling;
  if (tol<tolmin) tol=tolmin;
 
  a=0.0;
  b=(tol>1.0) ? tol : 1.0;
  fa=*fx;
  fb=funcval(optdata,origin,line,b);

  /* Ensure direction is downhill from 'a' to 'b' 
     If no slope is detected then scale up b-a an allowable number of times 
     to try and find one */

  iternum=0;
  while (fb==fa && ++iternum <= ctrl->maxiterbracket0)
    { b=b*2.0; 
      fb=funcval(optdata,origin,line,b);
    }

  if (fb==fa)
    { *xleft=a;
      *xright=b;
      *fleft=fa;
      *fleft=fb;
      *x=0.5*(a+b);
      if (logfile)
        { fprintf(logfile,
                 "Failed to find a slope; exceeded limit on iterations\n");
          fflush(logfile);
        }
      return 1;
    }

  else if (fb > fa) swap_points(&a,&b,&fa,&fb);

  /* Initialize c using golden step */

  c=b+GOLD*(b-a);
  fc=funcval(optdata,origin,line,c);

  /* Initialize counter for limiting iterations on a flat curve */

  stopcount=0;
  stop=0;

  /* While 'c' is downhill from 'b' (or level), check if parabola through a,b,c
     is concave. If not, take golden step to new trial point 'u', otherwise 
     construct parabola and and set 'u' to expected minimum or allowable limit 
     if step is too large relative to |c-b|. If 'u' is too close to 'c' 
     (on the basis of tolerance settings for 'minimize_brent') then it is set 
     a small distance beyond 'c'. Update a,b,c triplet with new point 'u'. 
     The algorithm is broadly similar to that presented in "Numerical Recipes 
     in C" Press et al. 1992. Although both algorithms may end up bracketing 
     local minima, an important difference is that the algorithm used here will
     not give up a known lower value in favour of a local minimum. */ 

  status=0;
  while (fc <= fb && (status=(stop || ++iternum > ctrl->maxiterbracket))==0)

    { /* Try new point u (tolerance is calculated as in 'minimize_brent' and 
         used to prevent the trial point being too close to 'c') */ 

      xtol=c;
      tol=ctrl->xfrctolbrent*fabs(xtol)+ctrl->xabstolbrent;
      tolmin=ctrl->xabstol*scaling;
      if (tol<tolmin) tol=tolmin;
 
      r=(b-a)*(fb-fc);
      q=(b-c)*(fb-fa);
      if (q<=r) 
        { parabolic=0;
          u=c+GOLD*(c-b);
        }
      else
        { parabolic=1;
          ulim=b+GLIMIT*(c-b);
          u=b-((b-c)*q-(b-a)*r)/(2.0*(q-r));
          if ((u-ulim)*(ulim-c)>0.0) u=ulim;
          else if (fabs(u-c)<tol) u=(c>b)? c+tol: c-tol;
        }
      fu=funcval(optdata,origin,line,u);

      /* Update a,b,c triplet 
         Stop search if trial point was beyond previous bound and new triplet 
         indicates a flat curve */

      a=b;
      fa=fb;
      if ((b-u)*(u-c)>0.0) 
        { b=u;
          fb=fu;
        }
      else
        { b=c;
          c=u;
          fb=fc;
          fc=fu;
          if (fb==fa && fc==fb && ++stopcount > ctrl->maxiterbracket0) stop=1;
          else stop=0;
        }

      if (logfile)
        { fprintf(logfile,
          "Bracketing iteration %d completed (parabolic=%d, tol=%g at x=%g):\n",
                  iternum,parabolic,tol,xtol); 
          prx=ceil(-log10(tol))+1;
          fprintf(logfile," %.*f %.*f %.*f | %g %g %g\n",
                  prx,a,prx,b,prx,c,fa,fb,fc);
        }
    }

  /* Output bracket points and low point */

  if (c > a) 
    { *xleft=a;
      *xright=c;
      *fleft=fa;
      *fright=fc;
    }
  else
    { *xleft=c;
      *xright=a;
      *fleft=fc;
      *fright=fa;
    }

  if (status==0)
    { *x=b;
      *fx=fb;
      if (logfile)
        { fprintf(logfile,"Minimum exists in interval: %g %g\n",*xleft,*xright);
          fflush(logfile);
        }
    }
  else
    { *x=c;
      *fx=fc;
       if (logfile)
         { if (stop)
             fprintf(logfile,
             "Failed to bracket a minimum; stopped with f(a)=f(b)=f(c)\n");
           else
             fprintf(logfile,
             "Failed to bracket a minimum; exceeded limit on iterations\n");
           fflush(logfile);
         }
    }

  return status;
}


static void swap_points (double *xa, double *xb, double *ya, double *yb)

{ double tmp;
  
  tmp=(*xa);
  *xa=(*xb);
  *xb=tmp;

  tmp=(*ya);
  *ya=(*yb);
  *yb=tmp;
}


/* minimize_brent: perform line minimization along the direction 'line' 
   through the point 'origin' (where we define x=0) using Brent's algorithm 
   (Brent 1973). A minimum is assumed to exist between 'xleft' and 'xright'.
   Values may optionally be supplied for the lowest point already found 'x' 
   and the function values at each of the 3 points. Setting all function 
   values to zero effectively de-selects this option. Termination conditions
   are controlled by tolerances 'xfrctolbrent', 'xabstolbrent' & 'ffrctolbrent'
   and the iteration limit 'maxiterbrent' in the 'optdata' record. 
   The fractional tolerance 'xfrctolbrent' should not generally be less than 
   the square root of the machine precision. The absolute tolerance term 
   'xabstolbrent' should be positive to ensure non-zero tolerance at x=0.
   'ffrctolbrent' is an optional fractional tolerance for the ordinate (can be 
   set to a non-zero value to speed up convergence). The tolerance on the 
   abcissa is adjusted if it is less than the global tolerance 'xabstol' scaled
   to 'x' units. ('x' is a multiple of the length of vector 'line'.) */

static int minimize_brent (struct powelldatarec *optdata, 
                           double origin[], double line[], double scaling,
                           double xleft, double xright, 
                           double fleft, double fright, 
                           double *x, double *fx) 

{ double a,b;          /* Lower and upper bound of search interval */
  double u;            /* Last evaluation point */
  double v;            /* Point with next lowest function value after w */
  double w;            /* Point with next lowest function value after x */
  double fu,fv,fw;     /* Function value at trial points */
  double m,p,q,r,d,e;
  double tol,tol2;     /* Tolerances */
  double tolmin;       /* Minimum tolerance required */
  int iternum;         /* Iteration number */
  int parabolic;       /* Flag for choosing between 
                          parabolic and golden section steps */
  double xtol; 
  int prx;
  FILE *logfile;
  struct powellctrlrec *ctrl;

  logfile=optdata->logfile;
  ctrl=optdata->ctrl;

  /* If a valid starting point is provided together with valid function values 
     at the initial bounds then initialize v,w to these bounds to save on 
     function evaluations, otherwise initialize as in Brent 1973. */

  if (*x > xleft && *x < xright && fleft > *fx && fright > *fx)
    { if (fleft > fright)
        { v=xleft;
          w=xright;
          fv=fleft;
          fw=fright;
        }
      else
        { v=xright;
          w=xleft;
          fv=fright;
          fw=fleft;
        }
      e=d=xright-xleft;  
    }
  else
    { v=w=(*x)=xleft+CGOLD*(xright-xleft);
      fv=fw=(*fx)=funcval(optdata,origin,line,*x);
      e=0.0;       
    } 

  /* Initialize bounds */

  a=xleft;
  b=xright;

  /* Loop until converged or iteration limit exceeded */

  iternum=0;

  while (1)

    { m=0.5*(a+b);

      /* Check convergence condition based on the fractional and
         absolute tolerances given for 'x' */

      xtol=*x;
      tol=ctrl->xfrctolbrent*fabs(xtol)+ctrl->xabstolbrent;
      tolmin=ctrl->xabstol*scaling;
      if (tol<tolmin) tol=tolmin;
      tol2=2*tol;

      if (fabs(*x-m) <= tol2-0.5*(b-a)) 
        { if (logfile)
            { fprintf(logfile,
                "Brent algorithm converged normally (tol=%g): f(x=%g)=%g\n",
                      tol,*x,*fx);
              fflush(logfile);
            }
          return 0;
        }

      /* Check whether function difference over 3 lowest points is within
         fractional tolerance given for Brent algorithm */

      if ((v != *x) && 2.0*(fv-(*fx)) < 
                       ctrl->ffrctolbrent*(fabs(*fx)+fabs(fv)))
        { if (logfile)
            { fprintf(logfile,
      "Brent algorithm converged with small change in ordinate: f(x)-f(v)=%g\n",
                      *fx-fv);
              fflush(logfile);
            }
          return 0;
        }

      /* Start new iteration */
 
      if (++iternum > ctrl->maxiterbrent) 
        { if (logfile) 
            { fprintf(logfile,
                      "Brent algorithm exceeded limit on iterations\n");
              fflush(logfile);
            }
          return 1;   
        }

      parabolic=0;

      if (fabs(e) > tol)
 
        { /* Construct parabola */ 

          r=(*x-w)*(*fx-fv);
          q=(*x-v)*(*fx-fw);
          p=(*x-v)*q-(*x-w)*r;
          q=2.0*(q-r);
          if (q > 0.0) p=-p;
          else q=-q;

          /* Check acceptability of p/q as the new step */ 

          if (fabs(p) < fabs(0.5*q*e) && p > q*(a-(*x)) && p < q*(b-(*x)))
            parabolic=1;
        }

      if (parabolic)

        { /* Use parabolic step but avoid getting too close to bounds */

          e=d; 
          d=p/q;
          u=(*x)+d;
          if (u-a < tol2 || b-u < tol2) d=(*x < m) ? tol : -tol;
        }

      else 

        { /* Use golden section step into largest sector */
 
          if (*x < m) e=b-(*x);
          else e=a-(*x);
          d=CGOLD*e; 
        }

      /* Check step is not too small and evaluate function at new point 'u' */

      if (fabs(d) < tol) d=(d > 0.0)? tol : -tol;
      u=(*x)+d;
      fu=funcval(optdata,origin,line,u);

      /* Update minimizer state */

      if (fu <= *fx)
        { if (u < *x) b=(*x);
          else a=(*x);
          v=w;
          w=(*x);
          *x=u;
          fv=fw;
          fw=(*fx);
          *fx=fu;
        }
      else
        { if (u < (*x)) a=u;
          else b=u;
          if (fu <= fw || w == (*x)) 
            { v=w;
              w=u;
              fv=fw;
              fw=fu;
            }
          else if (fu <= fv || v == (*x) || v == w)
            { v=u;
              fv=fu;
            }
        }

      if (logfile)
        { fprintf(logfile,
            "Brent iteration %d completed (parabolic=%d, tol=%g at x=%g):\n",
                  iternum,parabolic,tol,xtol);
          prx=ceil(-log10(tol))+1;
          fprintf(logfile," %.*f %.*f | %.*f %.*f %.*f | %g %g %g\n",
                  prx,a,prx,b,prx,v,prx,w,prx,*x,fv,fw,*fx);
        }
    }
}


/* funcval: return the objective function value at scalar displacement 'x' in 
   the direction 'line' from 'origin'. (Unit displacement is defined to be the 
   magnitude of the vector 'line'.) The optimizer's current parameter 
   vector is set to the corresponding location. */

static double funcval (struct powelldatarec *optdata,
                       double origin[], double line[], double x)

{ int i;
  double fval;
 
  for (i=0;i<optdata->n;i++) optdata->parmval[i]=origin[i]+x*line[i];
  fval=(*optdata->func)(optdata,optdata->parmval);
  
  if (optdata->logfile)
    fprintf(optdata->logfile,
            "Line minimization evaluation %d x,f(x): %g %.15g\n",
            optdata->evalnum,x,fval);

  return fval;
}
