!**************************************************************************!
! SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1 !
!                                                                          !
! Includes modifications to 2013 MarMOT 1.1 code (MarMOT 1.1.0) as listed  !
!  - Copyright 2015 John C. P. Hemmings                                    !
!**************************************************************************!
!**************************************************************************!
!                                                                          ! 
! SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013                     ! 
!                                                                          !
! Copyright Natural Environment Research Council, UK                       !
!                                                                          !
! Author: John Hemmings, National Oceanography Centre                      !
! e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013)                         !
! Alternative e-mail for author: marmot@wessexenv.mail1.co.uk              !
!                                                                          !
! This software is governed by the CeCILL  license under French law and    !
! abiding by the rules of distribution of free software.  You can  use,    ! 
! modify and/ or redistribute the software under the terms of the CeCILL   !
! license as circulated by CEA, CNRS and INRIA at the following URL        !
! "http://www.cecill.info".                                                ! 
!                                                                          !
! As a counterpart to the access to the source code and  rights to copy,   !
! modify and redistribute granted by the license, users are provided only  !
! with a limited warranty  and the software's author,  the holder of the   !
! economic rights,  and the successive licensors  have only  limited       !
! liability.                                                               ! 
!                                                                          !
! In this respect, the user's attention is drawn to the risks associated   !
! with loading,  using,  modifying and/or developing or reproducing the    !
! software by the user in light of its specific status of free software,   !
! that may mean  that it is complicated to manipulate,  and  that  also    !
! therefore means  that it is reserved for developers  and  experienced    !
! professionals having in-depth computer knowledge. Users are therefore    !
! encouraged to load and test the software's suitability as regards their  !
! requirements in conditions enabling the security of their systems and/or ! 
! data to be ensured and,  more generally, to use and operate it in the    ! 
! same conditions as regards security.                                     ! 
!                                                                          ! 
! The fact that you are presently reading this means that you have had     !
! knowledge of the CeCILL license and that you accept its terms.           !
!                                                                          !
!**************************************************************************!
!=======================================================================
!
! MODULE modsock
! John Hemmings - 12 Nov 2013
!
! Contains MarMOT routines for interfacing with the ecosystem model.
! Provides 'sockets' for plugging in model-specific routines.
!
! MarMOT generic model interface subroutines:
!
!  - modeldescrip - get model names and version details 
!  - modeldefparm - define model parameter set
!  - assimdefparm - define assimilation scheme parameter set
!  - modelgetinfo - get parameter-specific info about model for input checks
!  - modelsetup - set up fixed model environment (incl. option parameters) 
!  - assimsetup - set up assimilation scheme environment & parameters
!  - modelinit - initialize MarMOT tracer and ratio arrays    
!
!  - modelstep - do one standard time step
!     - modelgetbiotrans - get tracer-specific transport data 
!     - ecosystem - add ecosystem model tendencies to general tendency 
!         - modelgetphotin - get data for photosynthesis calc. 
!         - modeltrend - add now tendency from model sources minus sinks 
!         - getdiagnostics_biostep - get current values of ecosystem model 
!                                    diagnostics (multiple bio steps)
!     - assimilation - do data assimilation analysis and apply increments
!
!  - getdiagnostics - output model diagnostics
!     - getdiagnostics_biostep - as above (single bio step)
!
!  - getassimreport - output assimilation analysis report
!
! MODIFIED:
!
! John Hemmings - 18 Jan 2015 
! - Fixed type mismatches between reals of 'kind_r_ext' and 'kind_r'
! - Introduced 'kind_i_ext' and 'kind_c_ext' for integer and character
! types used in marmot.c interface 
! - Made all character type variables in marmot.c interface length 1 
! to ensure interoperability (Replaced old subroutine 'modelcodever' by
! 'modeldescrip' that includes the required conversion of model name 
! strings to character arrays for use by marmot.c) 
! - Introduced use of ISO C Binding features for portability 
! (suppressed if 'no_iso_c_binding' is defined)
! - Initialized 'reqsolar' flag for safety before call to 'hadocc_setup'
!
!=======================================================================

  module modsock

#ifndef no_iso_c_binding
  use iso_c_binding
#endif

  use modconfig, only: kind_r_ext,kind_i_ext,kind_c_ext,kind_r, &
                       objf_nodata,nk,nt,nr,ndp,nds,vertextlen

  use trlink, only: ntpri,ntracer,ntracer_in,itracer

  use og99npzd, only: & 
   og99npzd_codever => codever, &
   og99npzd_defparm => defparm, &
   og99npzd_getinfo => getinfo, &
   og99npzd_setup => setup, &
   og99npzd_setparm => setparm, &
   og99npzd_getbiotrans => getbiotrans, &
   og99npzd_getphotin => getphotin, &
   og99npzd_biotrend => biotrend, &
   og99npzd_getdiag => getdiag 

  use hadocc, only: &
   hadocc_codever => codever, &
   hadocc_defparm => defparm, & 
   hadocc_getinfo => getinfo, & 
   hadocc_setup => setup, &
   hadocc_setparm => setparm, &
   hadocc_initstate => initstate, &
   hadocc_getbiotrans => getbiotrans, &
   hadocc_getphotin => getphotin, &
   hadocc_biotrend => biotrend, &
   hadocc_getdiag => getdiag, &
   hadocc_ocda_defparm => ocda_defparm, &
   hadocc_ocda_setup => ocda_setup, &
   hadocc_ocda_setparm => ocda_setparm, &
   hadocc_ocda_analysis => ocda_analysis, &
   hadocc_ocda_applyinc => ocda_applyinc, &
   hadocc_ocda_getdiag => ocda_getdiag

  use medusa, only: &
   medusa_codever => codever, &
   medusa_defparm => defparm, &
   medusa_getinfo => getinfo, & 
   medusa_setup => setup, &
   medusa_setparm => setparm, &
   medusa_initstate => initstate, &
   medusa_getbiotrans => getbiotrans, &
   medusa_getphotin => getphotin, &
   medusa_biotrend => biotrend, &
   medusa_getdiag => getdiag

  implicit none

  integer,parameter :: np=500       ! Maximum number of model parameters

! The array sizes below are used for aggregate variable configuration.
! Any MarMOT tracer can potentially be defined as an aggregate tracer in
! the context of a particular model. Total primary production is treated
! as an aggregate diagnostic.

  integer,parameter :: ntagg=30     ! No. of potential aggregate tracers
  integer,parameter :: ndagg=1      ! No. of potential aggregate
                                    !   diagnostics
  integer,parameter :: nagg=6       ! Max number of primary tracers
                                    ! contributing to aggregate tracer
                                    ! or diagnostic

! MarMOT array indices (See 'modconfig' module for indices available to models)
! -- profile diagnostics array:

  integer,parameter :: ntder=17     ! No. of derived tracers
  integer,parameter :: ndpbasic=48  ! No. of profile diagnostics excluding 
                                    !   tracer-specific tendency group 

  integer,parameter :: dpoff_ddth=ndpbasic
                            ! Offset for implicit horizontal flow tendency
                            !   indices (1 tendency per tracer)

  integer,parameter :: dpoff_ddtp=dpoff_ddth+ntpri+ntder
                            ! Offset for perturbation tendency indices
                            !   (1 tendency per tracer)

  integer,parameter :: dpoff_ddtr=dpoff_ddtp+ntpri+ntder
                            ! Offset for relaxation tendency indices
                            !   (1 tendency per tracer)

! -- aggregate diagnostics array:

  integer,parameter :: i_pprod_agg=1   ! Primary production 

  ! Copies of parameter values for marmot.c

#ifdef no_iso_c_binding
  integer(kind_i_ext) :: modelnp=np      
  integer(kind_i_ext) :: modelntagg=ntagg      
  integer(kind_i_ext) :: modelndagg=ndagg      
  integer(kind_i_ext) :: modelnagg=nagg        
  integer(kind_i_ext) :: modelntder=ntder      
  integer(kind_i_ext) :: modelndpbasic=ndpbasic
#else
  integer(kind_i_ext),bind(C,name="FORTRAN_modelnp") :: modelnp=np 
  integer(kind_i_ext),bind(C,name="FORTRAN_modelntagg") :: modelntagg=ntagg 
  integer(kind_i_ext),bind(C,name="FORTRAN_modelndagg") :: modelndagg=ndagg 
  integer(kind_i_ext),bind(C,name="FORTRAN_modelnagg") :: modelnagg=nagg 
  integer(kind_i_ext),bind(C,name="FORTRAN_modelntder") :: modelntder=ntder 
  integer(kind_i_ext),bind(C,name="FORTRAN_modelndpbasic") :: &
    modelndpbasic=ndpbasic 
#endif

! Run configuration data

  integer ecomodel          ! Ecosystem model number
  integer photosmodel       ! Photosynthesis model number 
  integer attenmodel        ! Light attenuation model number
  integer attenpathopt      ! Light atten. path length adjust option
  real(kind_r) attenwater   ! Attenuation due to water (/m)
  real(kind_r) attenpig     ! Attenuation due to pigment (/m/(mg/m^3))
  integer achlmodel         ! Light absorption model number
  integer advscheme         ! Advection scheme number 
  logical advscheme_ctflag  ! Flag indicating scheme is centred in time
                            !   i.e. tendency calc. uses tracer values
                            !   at mid point of leapfrog double time step 
  integer diffscheme        ! Diffusion scheme number 
  integer mixscheme         ! Upper boundary layer mixing option
                            !   0 = No mixing
                            !   1 = Mix levels above MLD only
                            !   2 = Include partial mixing of level at MLD
  real(kind_r) latitude     ! Latitude (deg N)
  real(kind_r) zlevbot(nk)  ! Depth at bottom of level (m)
  real(kind_r) dz(nk)       ! Level height (m)
  real(kind_r) dzw(1:nk)    ! Level mid-point separation (m)
  integer km                ! Maximum depth level
  integer kmphotic          ! Maximum depth level for photosynthesis
  real(kind_r) timestep     ! Model time step (d) (= base time step)
  integer nstepbio          ! Number of sub-steps for biology
  logical leapfrog          ! Switch for leapfrog time-stepping
  logical initialstep       ! Switch to control leapfrog initialization 
  real(kind_r) pnu          ! Time-filter parameter for leapfrog scheme
  real(kind_r) pnu2m        ! 1-2*pnu
  real(kind_r) dtpert       ! Time interval between successive values in 
                            !   autoregressive process used in perturbations
  integer assimscheme       ! Data assimilation scheme number 
  integer assimstep         ! Step number for assimilation scheme analysis
  integer assimstepcount    ! Assimilation scheme step counter

! MarMOT tracer arrays - accessed using MarMOT indexing specified in 
! 'modconfig' module   

  real(kind_r_ext) tracer_mid(nk,nt)  
                            ! Tracers at mid point of double time step in 
                            !   leapfrog scheme 
  real(kind_r_ext) tracer_new(nk,nt) 
                            ! Working copy of tracers for new time step

! Mixing-related data available for diagnostics

  integer kmixed            ! Number of fully mixed layers       
  real(kind_r) dzmixed      ! Penetration of mixing into partially
                            !   mixed level (m)

! MarMOT diagnostic arrays - working copies, accessed using MarMOT indexing 
! specified in 'modconfig' module 

  real(kind_r_ext) diagprof_tmid(nk,ndp) ! Profiles at middle of std. time step
  real(kind_r_ext) diagscal_tmid(nds)    ! Scalars at middle of std. time step

! Photosynthesis model diagnostics (including intermediate variables)

  integer,parameter :: nphymax=2

  real(kind_r) limdin(nk,nphymax) ! DIN limitation factor 
  real(kind_r) vp(nk,nphymax)     ! Phytoplankton max. growth rate (/d)
  real(kind_r) kdpar(nk)          ! Diffuse attenuation coeff. for 
                                  !   downwelling PAR (/m)
  real(kind_r) parfraczbot(0:nk)  ! Bottom-of-level fraction of surface PAR
  real(kind_r) parzbot(0:nk)      ! Bottom-of-level PAR (E/m^2/d)
  real(kind_r) parfracz(nk)       ! Level mid-point fraction of surface PAR
  real(kind_r) parz(nk)           ! Level mid-point PAR (E/m^2/d)
  real(kind_r) alpha(nk,nphymax)  ! Initial slope of P-E curve for 
                                  !   downwelling PAR (/(E/m^2))
  real(kind_r) photz(nk,nphymax)  ! Level mid-point photosynthesis (/d)
  real(kind_r) photav(nk,nphymax) ! Level mean photosynthesis (/d)
  real(kind_r) rphotaeav(nk,nphymax)
                                  ! Level mean photosynthesis:(alpha*par) ratio
  real(kind_r) daylen             ! Day length (d) 
  real(kind_r) zeuphotic          ! Euphotic zone depth, 1% light (m)

! Relaxation intermediate available as diagnostic 

  real(kind_r) relaxratefrac      ! Fraction of input relaxation rate 
                                  !   permitted by tendency limitation 

! Tracer-specific velocities - accessed by MarMOT tracer indices

  real(kind_r) wbio(nk,nt)        ! Velocity relative to water (m/d, up)
  logical detsinkflag             ! Set to output detrital sinking diagnostic

! Tracer tendency diagnostics - accessed by MarMOT tracer indices 

  real(kind_r) ddth(nk,nt)        ! Horizontal influxes for activated 
                                  !   tracers (units/d)
  real(kind_r) ddtp(nk,nt)        ! Perturbation tendencies for activated 
                                  !   tracers (units/d)
  real(kind_r) ddtr(nk,nt)        ! Relaxation tendencies for activated 
                                  !   tracers (units/d)

! MODEL-SPECIFIC CONFIGURATION 

  integer,parameter :: nmod=3     ! Number of models

  ! Copy of parameter value for marmot.c & flags to activate ocean colour 
  ! assimilation for individual models

#ifdef no_iso_c_binding

  integer(kind_i_ext) :: nmodel=nmod  
  integer(kind_i_ext) :: modassimflag(nmod)=(/0,1,0/) 

#else

  integer(kind_i_ext),bind(C,name="FORTRAN_nmodel") :: nmodel=nmod 
  integer(kind_i_ext),bind(C,name="FORTRAN_modassimflag") :: &
    modassimflag(nmod)=(/0,1,0/) 

#endif

  integer,parameter :: og99npzdmodel=1
  integer,parameter :: hadoccmodel=2
  integer,parameter :: medusamodel=3

  character(len=vertextlen+1) :: modname_str(nmod)=(/ &
    'Oschlies & Garcon 1999 NPZD model              ', &
    'HadOCC, modified Palmer & Totterdell 2001 model', &
    'MEDUSA, Yool et al 2011 model                  '/)

! Note: the ecosystem model specifies the subsets of MarMOT tracers and ratios
! to be activated by setting elements of the index vectors 'itracer' and 
! 'iratio' to 'i_*' values defined in the 'modconfig' module. 
! ('itracer' is declared in module 'trlink'.)
! The index vectors are then used to access the MarMOT variables. 
! Tracer/ratio indices can be set to zero by the model to suppress members
! of the activated subsets not used in the current model configuration. 

  integer nratio_in    ! Number of ratios for model initialization
  integer iratio(nr)   ! MarMOT ratio indices for ratios used in init.

  real(kind_r) ratio_default(nr)
                       ! Default values for composition ratios 

  integer nphy         ! Number of phytoplankton types for MarMOT
                       ! photosynthesis models

  contains

!=======================================================================

#ifdef no_iso_c_binding

  subroutine modeldescrip(model,modname,version,vernote1,vernote2, &
                          description,descripnote1,descripnote2)

#else

  subroutine modeldescrip(model,modname,version,vernote1,vernote2, &
                          description,descripnote1,descripnote2) &
             bind(C,name="FORTRAN_modeldescrip")

#endif

  implicit none

! Input argument

  integer(kind_i_ext) model                ! Ecosystem model number

! Output arguments 

  character(kind=kind_c_ext), dimension(vertextlen+1) :: modname
                                           ! Model name 
  character(kind=kind_c_ext), dimension(vertextlen+1) :: version 
                                           ! Code version details 
  character(kind=kind_c_ext), dimension(vertextlen+1) :: vernote1  
                                           ! Code version note 1 
  character(kind=kind_c_ext), dimension(vertextlen+1) :: vernote2 
                                           ! Code version note 2 
  character(kind=kind_c_ext), dimension(vertextlen+1) :: description
                                           ! Model description ref.
  character(kind=kind_c_ext), dimension(vertextlen+1) :: descripnote1
                                           ! Model description note 1 
  character(kind=kind_c_ext), dimension(vertextlen+1) :: descripnote2 
                                           ! Model description note 2 

!-----------------------------------------------------------------------

  character(len=vertextlen+1) version_mod
  character(len=vertextlen+1) vernote1_mod
  character(len=vertextlen+1) vernote2_mod
  character(len=vertextlen+1) description_mod
  character(len=vertextlen+1) descripnote1_mod
  character(len=vertextlen+1) descripnote2_mod
    
  version_mod=""
  vernote1_mod=""
  vernote2_mod=""
  description_mod=""
  descripnote1_mod=""
  descripnote2_mod=""

  select case(model)
  case(og99npzdmodel)
    call og99npzd_codever(version_mod,description_mod)
  case(hadoccmodel)
    call hadocc_codever(version_mod,vernote1_mod,description_mod, &
                        descripnote1_mod)
  case(medusamodel)
    call medusa_codever(version_mod,description_mod)
  end select

  if (modname_str(model)/="") modname=transfer(modname_str(model),modname)
  if (version_mod/="") version=transfer(version_mod,version)
  if (vernote1_mod/="") vernote1=transfer(vernote1_mod,vernote1)
  if (vernote2_mod/="") vernote2=transfer(vernote2_mod,vernote2)
  if (description_mod/="") description=transfer(description_mod,description)
  if (descripnote1_mod/="") descripnote1=transfer(descripnote1_mod,descripnote1)
  if (descripnote2_mod/="") descripnote2=transfer(descripnote2_mod,descripnote2)

  end subroutine

!=======================================================================

#ifdef no_iso_c_binding

  subroutine modeldefparm(model,nparm,pname,pdesc,pmin,pmax)

#else

  subroutine modeldefparm(model,nparm,pname,pdesc,pmin,pmax) &
             bind(C,name="FORTRAN_modeldefparm")

#endif

  use modconfig, only: pnamelen,pdesclen 

  implicit none

! Input argument
      
  integer(kind_i_ext) model     ! Ecosystem model number 

! Output arguments

  integer(kind_i_ext) nparm     ! Number of parameters
  character(kind=kind_c_ext), dimension(pnamelen+1,np) :: pname  
                                ! Parameter names
  character(kind=kind_c_ext), dimension(pdesclen+1,np) :: pdesc 
                                ! Descriptions
  real(kind_r_ext) pmin(np)     ! Minimum values
  real(kind_r_ext) pmax(np)     ! Maximum values

!-----------------------------------------------------------------------

  integer nparm_mod
  character(len=pnamelen+1), dimension(np) :: pname_mod
  character(len=pdesclen+1), dimension(np) :: pdesc_mod  
  integer i

  nparm_mod=0
     
  select case(model)

  case(og99npzdmodel)
    call og99npzd_defparm(nparm_mod,pname_mod,pdesc_mod,pmin,pmax)
  case(hadoccmodel)
    call hadocc_defparm(nparm_mod,pname_mod,pdesc_mod,pmin,pmax)
  case(medusamodel)
    call medusa_defparm(nparm_mod,pname_mod,pdesc_mod,pmin,pmax)
  end select

  nparm=nparm_mod

  do i=1,nparm
    pname(:,i)=transfer(pname_mod(i),pname(:,i))
    pdesc(:,i)=transfer(pdesc_mod(i),pdesc(:,i))
  enddo

  end subroutine

!=======================================================================

#ifdef no_iso_c_binding

  subroutine assimdefparm(model,nparm,pname,pdesc,pmin,pmax)

#else

  subroutine assimdefparm(model,nparm,pname,pdesc,pmin,pmax) &
             bind(C,name="FORTRAN_assimdefparm")

#endif

  use modconfig, only: pnamelen,pdesclen

  implicit none

! Input argument

  integer(kind_i_ext) model     ! Ecosystem model number

! Output arguments

  integer(kind_i_ext) nparm     ! Number of parameters
  character(kind=kind_c_ext), dimension(pnamelen+1,np) :: pname  
                                ! Parameter names
  character(kind=kind_c_ext), dimension(pdesclen+1,np) :: pdesc 
                                ! Descriptions
  real(kind_r_ext) pmin(np)     ! Minimum values
  real(kind_r_ext) pmax(np)     ! Maximum values

!-----------------------------------------------------------------------

  integer nparm_mod
  character(len=pnamelen+1), dimension(np) :: pname_mod
  character(len=pdesclen+1), dimension(np) :: pdesc_mod  
  integer i
  
  nparm_mod=0
 
  select case(model)
  case(hadoccmodel)
    call hadocc_ocda_defparm(nparm_mod,pname_mod,pdesc_mod,pmin,pmax)
  end select

  nparm=nparm_mod

  do i=1,nparm
    pname(:,i)=transfer(pname_mod(i),pname(:,i))
    pdesc(:,i)=transfer(pdesc_mod(i),pdesc(:,i))
  enddo

  end subroutine

!=======================================================================
  
#ifdef no_iso_c_binding

  subroutine modelgetinfo(model,nparm,parm,photflag,latflag, &
                          advectflag,errmsg)

#else

  subroutine modelgetinfo(model,nparm,parm,photflag,latflag, &
                          advectflag,errmsg) &
             bind(C,name="FORTRAN_modelgetinfo")

#endif

  use modconfig, only: msgtextlen 

  implicit none

! Input argument

  integer(kind_i_ext) model        ! Ecosystem model number
  integer(kind_i_ext) nparm        ! Number of input parameters  
  real(kind_r_ext) parm(nparm)     ! Model-specific parameters

! Output arguments

  integer(kind_i_ext) photflag     ! Photosynthesis model flag
                                   ! -1: external phot model prohibited 
                                   ! 1: external phot model required 
                                   ! 2: external phot model with 
                                   !    phot:(alpha*par) ratio required 
  integer(kind_i_ext) latflag      ! Latitude flag (1 if required) 
  integer(kind_i_ext) advectflag   ! Advection scheme flag (1 if required) 
  character(kind=kind_c_ext), dimension(msgtextlen+1) :: errmsg 
                                   ! Error message

!-----------------------------------------------------------------------

  integer photflag_mod
  integer latflag_mod
  integer advectflag_mod
  character(len=msgtextlen+1) errmsg_mod  
  integer i

  photflag_mod=0
  latflag_mod=0
  advectflag_mod=0
  errmsg_mod=""  

  select case(model)

  case(og99npzdmodel)
    call og99npzd_getinfo(parm,advectflag_mod) 
  case(hadoccmodel)
    call hadocc_getinfo(parm,photflag_mod,latflag_mod,advectflag_mod,errmsg_mod)
  case(medusamodel)
    call medusa_getinfo(parm,photflag_mod,latflag_mod,advectflag_mod,errmsg_mod)
  end select

  photflag=photflag_mod
  latflag=latflag_mod
  advectflag=advectflag_mod
  
  if(errmsg_mod/="") errmsg=transfer(errmsg_mod,errmsg)

  end subroutine

!=======================================================================

#ifdef no_iso_c_binding

  subroutine modelsetup(model,nparm,parm,photmodel,kdmodel, &
    kdpathopt,kwater,kpig,amodel,advectopt,diffopt,mixopt, &
    pmeanflag,pranflag,ptrans,pautocorr,pseed,pmixopt, &
    rlxflag,rlxctrlfact,rlxmaskflag,rlxmaskopt,rlxmixopt, &
    lat,kmax,zbot_m,tstep,nstepb,lfrog,tfparm, &
    nt_nullmod,itracer_nullmod,it_agg,id_agg, &
    reqsol,reqsolav,reqmld,reqsst,reqsss,reqpco2atm,reqdust, &
    reqfedust,reqtemp,reqw,reqvdc,dsinkext) 

#else

  subroutine modelsetup(model,nparm,parm,photmodel,kdmodel, &
    kdpathopt,kwater,kpig,amodel,advectopt,diffopt,mixopt, &
    pmeanflag,pranflag,ptrans,pautocorr,pseed,pmixopt, &
    rlxflag,rlxctrlfact,rlxmaskflag,rlxmaskopt,rlxmixopt, &
    lat,kmax,zbot_m,tstep,nstepb,lfrog,tfparm, &
    nt_nullmod,itracer_nullmod,it_agg,id_agg, &
    reqsol,reqsolav,reqmld,reqsst,reqsss,reqpco2atm,reqdust, &
    reqfedust,reqtemp,reqw,reqvdc,dsinkext) &
    bind(C,name="FORTRAN_modelsetup")

#endif

  use modconfig, only: i_din,i_phy,i_pndia,i_pdiat,i_zoo,i_zmi,i_zme,i_det, &
                       i_dic,i_chl,i_phyc,i_zooc,i_dsi,i_dfe, &
                       i_nitrogen,i_carbon,i_silicon,i_iron,i_pon,i_poc
  use trlink, only: ntracer_linked,ibasetracer,linkratio_def,linkratio_min, &
                    linkratio_max
  use trperturb, only: pertsetup
  use trrelax, only: relaxsetup,relaxactive
  use photos, only: photsetup
 
  implicit none

! Input arguments 

  integer(kind_i_ext) model       ! Ecosystem model number 
  integer(kind_i_ext) nparm       ! Number of input parameters  
  real(kind_r_ext) parm(nparm)    ! Model-specific parameters
  integer(kind_i_ext) photmodel   ! Photosynthesis model number 
  integer(kind_i_ext) kdmodel     ! Light attenuation model number 
  integer(kind_i_ext) kdpathopt   ! Light atten. path length adjust option 
  real(kind_r_ext) kwater         ! Light attenuation due to water (/m)
  real(kind_r_ext) kpig           ! Atten. due to pigment (/m/(mg/m^3))
  integer(kind_i_ext) amodel      ! Light absorption model number
  integer(kind_i_ext) advectopt   ! Advection scheme number
  integer(kind_i_ext) diffopt     ! Diffusion scheme number
  integer(kind_i_ext) mixopt      ! Mixing option for boundary layer
                                  !   0: no mixing
                                  !   1: mix fully mixed layers only
                                  !   2: part mix layer at ref. depth
  integer(kind_i_ext) pmeanflag(ntpri)        ! Tracer perturbation flag  
                                  !   0: no prescribed perturbation
                                  !   1: presecribed perturbation active 
  integer(kind_i_ext) pranflag(ntpri)         ! Tracer perturbation flag 
                                  !   0: no random perturbation
                                  !   1: random perturbation active
  integer(kind_i_ext) ptrans(ntpri)
                                  ! Tracer perturbation transformation 
                                  !   0: no variable transformation 
                                  !   1: log10 transformation 
                                  !   2: sqrt transformation 
  real(kind_r_ext) pautocorr      ! Perturbation autocorrelation after 24 h
  integer(kind_i_ext) pseed(ntpri)
                                  ! Tracer perturbation seed for random  
                                  !   number generation 
  integer(kind_i_ext) pmixopt     ! Mixing option after perturbation (0-2)
  integer(kind_i_ext) rlxflag(ntpri)
                                  ! Tracer relaxation flag
                                  !   0: no relaxation
                                  !   1: relaxation active
  real(kind_r_ext) rlxctrlfact    ! Multiple of perturbation s.d. for 
                                  ! limiting relaxation rate
  integer(kind_i_ext) rlxmaskflag ! Tracer relaxation mask flag 
                                  !  -1: mask levels at z or above
                                  !   0: no mask
                                  !   1: mask levels at z or below
  integer(kind_i_ext) rlxmaskopt  ! Tracer relaxation mask option
                                  !   1: z = upper mixed layer depth
                                  !   2: z = euphotic zone depth 
                                  !   3: z = max. of (1) and (2) 
  integer(kind_i_ext) rlxmixopt   ! Mixing option after relaxation (0-2)
  real(kind_r_ext) lat            ! Latitude (deg N)
  integer(kind_i_ext) kmax        ! Maximum depth level to process 
  real(kind_r_ext) zbot_m(nk)     ! Depth at bottom of level (m)
  real(kind_r_ext) tstep          ! Model timestep (d)
  integer(kind_i_ext) nstepb      ! Number of sub-steps for biology
  integer(kind_i_ext) lfrog       ! Flag for leapfrog scheme
  real(kind_r_ext) tfparm         ! Robert-Asselin time filter parameter 
  integer(kind_i_ext) nt_nullmod  ! Number of tracers for null model
  integer(kind_i_ext) itracer_nullmod(nt)   
                                  ! Indices of null model tracers

! Output arguments

  integer(kind_i_ext) it_agg(nagg,ntagg)  
                                  ! Indices of primary tracers contributing
                                  !   to aggregate tracers
  integer(kind_i_ext) id_agg(nagg,ndagg) 
                                  ! Indices of primary tracers contributing
                                  !   to aggregate diagnostics
  integer(kind_i_ext) reqsol      ! Flag requesting pt-in-time solar rad.
  integer(kind_i_ext) reqsolav    ! Flag requesting daily mean solar rad.
  integer(kind_i_ext) reqmld      ! Flag requesting mixed layer depth
  integer(kind_i_ext) reqsst      ! Flag requesting sea-surface temp.
  integer(kind_i_ext) reqsss      ! Flag requesting sea-surface salinity
  integer(kind_i_ext) reqpco2atm  ! Flag requesting atmospheric pCO2
  integer(kind_i_ext) reqdust     ! Flag requesting dust deposition 
  integer(kind_i_ext) reqfedust   ! Flag requesting aeolian iron input 
  integer(kind_i_ext) reqtemp     ! Flag requesting full-depth temp.
  integer(kind_i_ext) reqw        ! Flag requesting vertical velocity 
  integer(kind_i_ext) reqvdc      ! Flag requesting vert. diffusion coeff. 
  integer(kind_i_ext) dsinkext    ! Flag indicating external handling of 
                                  !   detrital sinking by MarMOT advection 
                                  !   scheme (for MarMOT diagnostics) 

!-----------------------------------------------------------------------

  integer reqsol_mod
  integer reqsolav_mod
  integer reqmld_mod
  integer reqsst_mod
  integer reqsss_mod
  integer reqpco2atm_mod
  integer reqdust_mod
  integer reqfedust_mod
  integer reqtemp_mod
  integer dsinkext_mod
  integer reqsolar
  integer i,j,k

! Make global copies of input data

  advscheme=advectopt
  diffscheme=diffopt
  mixscheme=mixopt
  ecomodel=model
  photosmodel=photmodel
  attenmodel=kdmodel
  attenpathopt=kdpathopt
  achlmodel=amodel
  latitude=lat

! Initialize forcing variable request flags and advection scheme configuration.
! The upstream scheme and MUSCL schemes are forward-in-time schemes. 
! There are no time-centred schemes implemented in this version of the code.

  if (advectopt > 0) then
    reqw=1
    select case(advectopt)
    case(1)                    ! Upstream scheme 
      advscheme_ctflag=.FALSE.
    case(2)                    ! MUSCL scheme
      advscheme_ctflag=.FALSE.
    end select
  else 
    reqw=0
  endif

  if (diffopt > 0) then
    reqvdc=1
  else 
    reqvdc=0
  endif

  if (mixopt>0 .or. pmixopt>0 .or. rlxmixopt>0) then
    reqmld=1
  else
    reqmld=0
  endif

  reqsol=0
  reqsolav=0
  reqtemp=0
  reqsst=0
  reqsss=0
  reqpco2atm=0
  reqdust=0
  reqfedust=0
  dsinkext=0

! Set up grid-related data 

  km=kmax
  if (km>nk) km=nk

  zlevbot(1)=zbot_m(1)
  dz(1)=zlevbot(1)
  do k=2,km
    zlevbot(k)=zbot_m(k)
    dz(k)=zlevbot(k)-zlevbot(k-1)
  enddo

  if (diffscheme/=0) then

    do k=1,km-1
      dzw(k)=0.5_kind_r*(dz(k)+dz(k+1))
    enddo

  endif

! Set up time steppping 

  timestep=tstep
  nstepbio=nstepb
  dtpert=0.0
  if (lfrog==0) then
    leapfrog=.FALSE.
    initialstep=.FALSE.  
  else
    leapfrog=.TRUE.
    initialstep=.TRUE.
  endif

  if (leapfrog) then
    pnu=tfparm
    pnu2m=1-2*tfparm
  endif 

! Initialize diagnostics workspace (if required)

  if (nstepbio>1) then
    diagprof_tmid(1:km,:)=objf_nodata
    diagscal_tmid(:)=objf_nodata
  endif 

! Set up the MarMOT photosynthesis model (if required)

  if (photosmodel>0) then
    call photsetup(km,dz,zlevbot,attenmodel,achlmodel,kmphotic)    
    attenwater=kwater
    attenpig=kpig
  endif

! Set up the main model

! Note re state variable mapping:
!
!   'ntracer' is the number of tracers to be processsed by the transport 
!      routines, including both initializable tracers and linked tracers.
!   'ntracer_in' is the number of initializable tracers (a subset of 
!      MarMOT primary tracers).
!   'ntracer_linked' is the number of additional tracers that are linked 
!      to MarMOT primary tracers (a subset of MarMOT secondary tracers).
!      Ratios of linked tracers to their base primary tracers are preserved
!      in perturbation and relaxation updates.
!   'ibasetracer' is a vector of 'ntracer_linked' model tracer 
!      indices specifying the tracers that are linked to. 
!   'itracer' is a vector of 'ntracer' MarMOT tracer indices, accessed by 
!      model tracer indices, with initializable tracers appearing first. 
!   'nratio_in' is the number of initializable ratios (a subset of 
!      MarMOT ratios).
!   'iratio' is a vector of 'nratio_in' MarMOT ratio indices. 
!
! Initializable tracers/ratios are those that the model expects to be
! initialized by MarMOT. The model can provide default values for the 
! initializable ratios and may initialize other MarMOT ratios itself. 
! Initializable tracers default to 0.  
! Processing of individual tracers/ratios is suppressed by setting index to 0.

  nratio_in=0
  ntracer_linked=0
  nphy=0

  if (ecomodel==0) then

!   In the absence of an ecosystem model activate all initialized tracers 

    ntracer_in=nt_nullmod
    itracer=itracer_nullmod

  else

!   Initialize number of tracers 

    ntracer_in=0

!   Initialize aggregate tracer & diagnostic definitions

    it_agg(1:nagg,1:ntagg)=0 
    id_agg(1:nagg,1:ndagg)=0 

!   Initialize default ratios

    ratio_default(1:nr)=0.0
 
!   Initialize photosynthesis model forcing variable request flags

    if (photmodel==2) reqsol=1 
    if (photmodel==1 .or. photmodel==3) reqsolav=1

!   Set-up ecosystem model

    reqsol_mod=reqsol
    reqsolav_mod=reqsolav
    reqmld_mod=reqmld
    reqsst_mod=reqsst
    reqsss_mod=reqsss
    reqpco2atm_mod=reqpco2atm
    reqdust_mod=reqdust
    reqfedust_mod=reqfedust
    reqtemp_mod=reqtemp
    dsinkext_mod=dsinkext

    select case(ecomodel)

    case(og99npzdmodel)

      call og99npzd_setup(km,ntracer_in,itracer,nphy,reqtemp_mod,dsinkext_mod)
      call og99npzd_setparm(parm) 

      it_agg(1,i_nitrogen)=i_din
      it_agg(2,i_nitrogen)=i_phy
      it_agg(3,i_nitrogen)=i_zoo
      it_agg(4,i_nitrogen)=i_det
      it_agg(1,i_pon)=i_phy
      it_agg(2,i_pon)=i_zoo
      it_agg(3,i_pon)=i_det
      it_agg(1:3,i_poc)=it_agg(1:3,i_pon)
      id_agg(1,i_pprod_agg)=i_phy

    case(hadoccmodel)

      reqsolar=0

      call hadocc_setup(km,zbot_m,lat,parm, &
             ntracer_in,ntracer_linked,ibasetracer,itracer,nratio_in,iratio, &
             nphy,reqsolar,reqmld_mod,reqtemp_mod,reqsst_mod,reqsss_mod, &
             reqpco2atm_mod,dsinkext_mod) 

      if (photmodel==0 .and. reqsolar==1) reqsolav=1  

      call hadocc_setparm(parm,ratio_default,linkratio_def,linkratio_min, &
                          linkratio_max) 

      it_agg(1,i_nitrogen)=i_din
      it_agg(2,i_nitrogen)=i_phy
      it_agg(3,i_nitrogen)=i_zoo
      it_agg(4,i_nitrogen)=i_det
      it_agg(1,i_carbon)=i_dic
      it_agg(2:4,i_carbon)=it_agg(2:4,i_nitrogen)
      it_agg(1,i_pon)=i_phy
      it_agg(2,i_pon)=i_zoo
      it_agg(3,i_pon)=i_det
      it_agg(1:3,i_poc)=it_agg(1:3,i_pon)
      id_agg(1,i_pprod_agg)=i_phy

    case(medusamodel)

      call medusa_setup(km,zbot_m,lat,parm, &
             ntracer_in,ntracer_linked,ibasetracer,itracer,nratio_in,iratio, &
             nphy,reqsol_mod,reqdust_mod,reqfedust_mod,reqtemp_mod,dsinkext_mod)

      call medusa_setparm(parm,ratio_default,linkratio_def,linkratio_min, &
                          linkratio_max) 

      it_agg(1,i_phy)=i_pndia
      it_agg(2,i_phy)=i_pdiat
      it_agg(1,i_zoo)=i_zmi
      it_agg(2,i_zoo)=i_zme
      it_agg(1,i_chl)=i_pndia
      it_agg(2,i_chl)=i_pdiat
      it_agg(1,i_phyc)=i_pndia
      it_agg(2,i_phyc)=i_pdiat
      it_agg(1,i_zooc)=i_zmi
      it_agg(2,i_zooc)=i_zme
      it_agg(1,i_nitrogen)=i_din
      it_agg(2,i_nitrogen)=i_pndia
      it_agg(3,i_nitrogen)=i_pdiat
      it_agg(4,i_nitrogen)=i_zmi
      it_agg(5,i_nitrogen)=i_zme
      it_agg(6,i_nitrogen)=i_det
      it_agg(1,i_carbon)=i_dic
      it_agg(2:6,i_carbon)=it_agg(2:6,i_nitrogen) 
      it_agg(1,i_silicon)=i_dsi
      it_agg(2,i_silicon)=i_pdiat
      it_agg(1,i_iron)=i_dfe
      it_agg(2,i_iron)=i_pndia
      it_agg(3,i_iron)=i_pdiat
      it_agg(4,i_iron)=i_zmi
      it_agg(5,i_iron)=i_zme
      it_agg(6,i_iron)=i_det
      it_agg(1,i_pon)=i_pndia
      it_agg(2,i_pon)=i_pdiat
      it_agg(3,i_pon)=i_zmi
      it_agg(4,i_pon)=i_zme
      it_agg(5,i_pon)=i_det
      it_agg(1:5,i_poc)=it_agg(1:5,i_pon)
      id_agg(1,i_pprod_agg)=i_pndia
      id_agg(2,i_pprod_agg)=i_pdiat

    end select

    reqsol=reqsol_mod
    reqsolav=reqsolav_mod
    reqmld=reqmld_mod
    reqsst=reqsst_mod
    reqsss=reqsss_mod
    reqpco2atm=reqpco2atm_mod
    reqdust=reqdust_mod
    reqfedust=reqfedust_mod
    reqtemp=reqtemp_mod
    dsinkext=dsinkext_mod

  endif

  ntracer=ntracer_in+ntracer_linked

! Zero tracer-specific vertical velocity profiles for activated tracers
! and set flag for diagnostic output

  detsinkflag=.FALSE.
  do i=1,ntracer
    j=itracer(i)
    if (j/=0) then
      wbio(1:km,j)=0.0
      if (j==i_det .and. ecomodel/=0) detsinkflag=.TRUE.
    endif
  enddo

! Zero tendency diagnostics for all activated tracers 

  do i=1,ntracer
    j=itracer(i)
    if (j/=0) then
      ddth(1:km,j)=0.0
      ddtp(1:km,j)=0.0
      ddtr(1:km,j)=0.0
    endif
  enddo

! Set up perturbation scheme 

  call pertsetup(pmeanflag,pranflag,ptrans,pautocorr,pseed,pmixopt)

! Set tracer relaxation control flags 

  call relaxsetup(rlxflag,rlxctrlfact,rlxmaskflag,rlxmaskopt,rlxmixopt) 
      
! Request MLD if needed for relaxation mask

  if (relaxactive .and. rlxmaskflag/=0 .and. &
      (rlxmaskopt==1 .or. rlxmaskopt==3)) reqmld=1

  end subroutine

!=======================================================================

#ifdef no_iso_c_binding

  subroutine assimsetup(scheme,nparm,parm,nstep,startstep, &
                        diagflag,diagfdflag,reqmld,reqcchl,reqchl, &
                        reqpgrow,reqploss)

#else

  subroutine assimsetup(scheme,nparm,parm,nstep,startstep, &
                        diagflag,diagfdflag,reqmld,reqcchl,reqchl, &
                        reqpgrow,reqploss) &
             bind(C,name="FORTRAN_assimsetup")

#endif

  implicit none

! Input arguments

  integer(kind_i_ext) scheme     ! Assimilation scheme number
  integer(kind_i_ext) nparm      ! Number of input parameters
  real(kind_r_ext) parm(nparm)   ! Assimilation scheme parameters
  integer(kind_i_ext) nstep      ! Number of model time steps between 
                                 !   analyses (= model steps per 
                                 !   assimilation time step)
  integer(kind_i_ext) startstep  ! Model step at start of first 
                                 !   assimilation time step 
  integer(kind_i_ext) diagflag   ! Flag for scalar diagnostics
  integer(kind_i_ext) diagfdflag ! Flag for profile diagnostics

! In/out argument
 
  integer(kind_i_ext) reqmld     ! Flag requesting mixed layer depth

! Output arguments

  integer(kind_i_ext) reqcchl    ! Flag requesting surface C:Chl 
  integer(kind_i_ext) reqchl     ! Flag requesting surface chlorophyll 
  integer(kind_i_ext) reqpgrow   ! Flag requesting phytoplankton
                                 !   specific growth rate 
  integer(kind_i_ext) reqploss   ! Flag requesting phytoplankton 
                                 !   specific loss rate 

!-----------------------------------------------------------------------

  integer nlead                ! Number of model steps before start of
                               !   first assimilation time step
  integer nstepskip            ! Number of 'modeltrend' calls before
                               !   start of first assimilation time step
  integer diagflag_mod
  integer diagfdflag_mod
  integer reqmld_mod
  integer reqcchl_mod
  integer reqchl_mod
  integer reqpgrow_mod
  integer reqploss_mod

  assimscheme=scheme
  reqcchl=0
  reqchl=0
  reqpgrow=0
  reqploss=0

  if (assimscheme==1) then

!   Set timing control parameters 

    nlead=startstep-1
    nstepskip=nlead*nstepbio

!   Adjust number of leading 'modeltrend' calls to allow for 
!   leapfrog initialization step

    if (leapfrog) then
      if (nstepbio == 1) then
        nstepskip=nstepskip+1
      else
        nstepskip=nstepskip+nstepbio/2
      endif
    endif

!   Do model-specific setup

    diagflag_mod=diagflag
    diagfdflag_mod=diagfdflag
    reqmld_mod=reqmld
    reqcchl_mod=reqcchl
    reqchl_mod=reqchl
    reqpgrow_mod=reqpgrow
    reqploss_mod=reqploss

    select case(ecomodel)
    case(hadoccmodel)

      call hadocc_ocda_setup(nstepskip,parm,diagflag_mod,diagfdflag_mod, &
                             reqmld_mod,reqcchl_mod,reqchl_mod,reqpgrow_mod, &
                             reqploss_mod)

      call hadocc_ocda_setparm(parm)

      assimstep=nstep
      assimstepcount=-nlead

    end select

    reqmld=reqmld_mod
    reqcchl=reqcchl_mod
    reqchl=reqchl_mod
    reqpgrow=reqpgrow_mod
    reqploss=reqploss_mod

  endif

  end subroutine

!=======================================================================

#ifdef no_iso_c_binding

  subroutine modelinit(tracer_ic,ratio_ic,tracer,ratio)

#else

  subroutine modelinit(tracer_ic,ratio_ic,tracer,ratio) &
             bind(C,name="FORTRAN_modelinit")

#endif

  implicit none

! Input arguments

  real(kind_r_ext) tracer_ic(nk,nt)  ! Tracer initial condition
  real(kind_r_ext) ratio_ic(nk,nr)   ! Ratio initial condition

! Output arguments

  real(kind_r_ext) tracer(nk,nt)     ! MarMOT tracers
  real(kind_r_ext) ratio(nk,nr)      ! MarMOT composition ratios

!-----------------------------------------------------------------------
! The model state is initialized from the external primary tracer 
! concentrations and variable composition ratios (if applicable). 
! Required external ratios are replaced by model defaults if missing.
! 
! After initialization, the prognostic state variables are typically 
! the primary and ratio-derived tracer concentrations. Ratios can 
! be used as prognostic variables but are not updated by MarMOT. 
! (Any time-step dependency in ratio updates must be handled by the model).
!
! Model-specific initialization routines apply any model constraints 
! to the input ratios and output the prognostic tracers and any 
! prognostic ratios. They may also output fixed composition ratios 
! for use in diagnostic calculations.
!
! N.B. MarMOT assumes all tracers initialized here are prognostic and
! will diagnose relevant output ratios from these tracers, replacing 
! any previous values.  
!-----------------------------------------------------------------------

  integer i,j,k

! Copy all input values expected by the current ecosystem model 
! (or use defaults if missing) 

  do i=1,ntracer_in
    j=itracer(i)
    if (j/=0) then
      do k=1,km
        if (tracer_ic(k,j)==objf_nodata) then
          tracer(k,j)=0.0
        else 
          tracer(k,j)=tracer_ic(k,j)
        endif
      enddo
    endif
  enddo

  do i=1,nratio_in
    j=iratio(i)
    if (j/=0) then
      do k=1,km
        if (ratio_ic(k,j)<0.0) then
          ratio(k,j)=ratio_default(i)
        else 
          ratio(k,j)=ratio_ic(k,j)
        endif
      enddo
    endif
  enddo

! Get any derived tracers and/or fixed composition ratios from the
! ecosystem model 

  select case(ecomodel)
  case(hadoccmodel) 
    call hadocc_initstate(tracer,ratio)
  case(medusamodel) 
    call medusa_initstate(tracer,ratio)
  end select

  end subroutine

!=======================================================================

#ifdef no_iso_c_binding

  subroutine modelstep(tofy,yearlen,f_sol,f_solav,f_mld,f_sss, &
                       f_pco2atm,f_dust,f_fedust,f_temp,f_vdc,f_w, &
                       f_pert,f_pertsd,f_rlx,f_ref0,f_ref1,f_cchl,f_chl, &
                       f_pgrow,f_ploss,tracer,ratio,assimnext,assimupd)

#else

  subroutine modelstep(tofy,yearlen,f_sol,f_solav,f_mld,f_sss, &
                       f_pco2atm,f_dust,f_fedust,f_temp,f_vdc,f_w, &
                       f_pert,f_pertsd,f_rlx,f_ref0,f_ref1,f_cchl,f_chl, &
                       f_pgrow,f_ploss,tracer,ratio,assimnext,assimupd) &
             bind(C,name="FORTRAN_modelstep")

#endif

  use tradvect, only: advection
  use trperturb, only: perturbation,pertactive,pertmixscheme,pertflag
  use trrelax, only: relaxation,override,relaxactive,relaxmixscheme, &
                     relaxflag,relaxctrlfact
  use trdiffuse, only: diffusion
  use trmix, only: mixinglevels,mixing,mixsubset 

  implicit none

! Input arguments

  real(kind_r_ext) tofy            ! Time of year (d)
  integer(kind_i_ext) yearlen      ! Year length (d)
  real(kind_r_ext) f_sol           ! Surface solar irradiance (W/m^2)  
  real(kind_r_ext) f_solav         ! Daily mean surface irradiance (W/m^2)
  real(kind_r_ext) f_mld           ! Upper mixed layer depth (m) 
  real(kind_r_ext) f_sss           ! Sea surface salinity 
  real(kind_r_ext) f_pco2atm       ! Atmospheric pCO2 (uatm)
  real(kind_r_ext) f_dust          ! Aeolian dust deposition (mg/m^2/d) 
  real(kind_r_ext) f_fedust        ! Aeolian soluble iron (mmolFe/m^2/d) 
  real(kind_r_ext) f_vdc(nk)       ! Vertical diffusion coefficient (m^2/d)
  real(kind_r_ext) f_w(nk)         ! Vertical vel (m/d)
  real(kind_r_ext) f_temp(nk)      ! Temperature (deg C) 
  real(kind_r_ext) f_pert(nk,ntpri)
                                   ! Perturb rates (trans. units/d) 
  real(kind_r_ext) f_pertsd(nk,ntpri)
                                   ! Perturb s.d.s (trans. units/d) 
  real(kind_r_ext) f_rlx(nk,ntpri) ! Relaxation rates (/d) 
  real(kind_r_ext) f_ref0(nk,ntpri) ! Reference tracer profiles, start of step
  real(kind_r_ext) f_ref1(nk,ntpri) ! Reference tracer profiles, end of step
  real(kind_r_ext) f_cchl          ! Surface C:Chl (gC/gChl) 
  real(kind_r_ext) f_chl           ! Surface chlorophyll (mg/m3)
  real(kind_r_ext) f_pgrow         ! Mixed layer phytoplankton specific 
                                   ! growth rate (\d) 
  real(kind_r_ext) f_ploss         ! Mixed layer phytoplankton specific 
                                   ! loss rate (\d) 

! In/out arguments

  real(kind_r_ext) tracer(nk,nt)   ! MarMOT tracers
  real(kind_r_ext) ratio(nk,nr)    ! MarMOT composition ratios

! Output argument 

  integer(kind_i_ext) assimnext    ! Flag requesting assimilation
                                   !   forcing data for next step
  integer(kind_i_ext) assimupd     ! Flag indicating an assimilation
                                   !   update has been done

!-----------------------------------------------------------------------
! Do one model time step.
!
! Time stepping is based on either a forward Euler scheme or a leapfrog
! scheme with a Robert-Asselin time filter. In both cases diffusion is 
! implemented by an implicit scheme.
! The model is advanced by 1 base time step.
!
! Forward Euler scheme.
! Standard time step length is equal to the base time step. 
!
! Leapfrog scheme.
! Standard time step length is twice the base time step. 
! 3 time-levels are maintained: 
!  - 'before' state Xn-1 ('tracer') is the external MarMOT tracer array 
!  - 'now' state Xn ('tracer_mid') is at the mid point of the double time step 
!  - 'after' state Xn+1 ('tracer_new') is at the end of the double time step 
! Model output Xnout at each base time step is a weighted average of the now 
! state and an interpolated state determined from the before and after states
! 
!    Xnout = (2 * pnu - 1) * Xn + pnu * (Xn-1 + Xn+1)
!
! Xn is used to calculate the tendencies for determining Xn+1 from Xn-1
! except where this is undesirable for stability reasons.
! Initially Xn is determined by a forward Euler step.
!
! Time-splitting scheme.
! Multiple sub-steps may be used in 'ecosystem' routine to calculate
! mean tendencies over the standard time step. Transport tendencies 
! and photosynthetic rate (unless internal to ecosystem model) are
! evaluated once only during each standard time step.
!
! Data assimilation updates are based on the tracer values destined
! for output at the current time step. In the leapfrog case they 
! are applied at two time levels.
!-----------------------------------------------------------------------

  real(kind_r_ext) ddt(km,nt)      ! Tracer tendencies (tracer units /d)
                                   !  - accessed by MarMOT tracer indices
                                   !  - declared with external type to
                                   !    allow for potential use in 'marmot.c'
  real(kind_r) dtstd               ! Standard time step (d) 
  real(kind_r) dtbio               ! Biological time step (d)   
  integer nstepbio_lf
  integer relaxkmin,relaxkmax
  logical flag(ntpri)
  integer i,j

! Identify levels within a mixed boundary layer; 
! Adjust perturbation & relaxation drivers for consistency if required 

  if (mixscheme > 0 .or. pertmixscheme > 0 .or. relaxmixscheme > 0) &
    call mixinglevels(km,zlevbot,f_mld,kmixed,dzmixed)

  if (pertmixscheme > 0) then
    call mixsubset(pertmixscheme,kmixed,dz,dzmixed,ntracer_in,itracer, &
                   pertflag,f_pert)
    call mixsubset(pertmixscheme,kmixed,dz,dzmixed,ntracer_in,itracer, &
                   pertflag,f_pertsd)
  endif

  if (relaxmixscheme > 0) then
    call mixsubset(relaxmixscheme,kmixed,dz,dzmixed,ntracer_in,itracer, &
                   relaxflag,f_ref0)
    call mixsubset(relaxmixscheme,kmixed,dz,dzmixed,ntracer_in,itracer, &
                   relaxflag,f_ref1)
    if (relaxctrlfact > 0.0) then
      flag=relaxflag.and.(.not.pertflag)
      call mixsubset(relaxmixscheme,kmixed,dz,dzmixed,ntracer_in,itracer, &
                     flag,f_pertsd)
    endif
  endif

! Set up time-stepping

  if (leapfrog) then

    if (initialstep) then

!     We do a forward Euler time step to get started so
!     adjust time step and use the 'before' tracer values for all 
!     tendency calculations by making 'now' values the same

      dtstd=timestep

      if (nstepbio == 1) then
        nstepbio_lf=1
      else
        nstepbio_lf=nstepbio/2
        if (nstepbio_lf*2 /= nstepbio) stop &
          "*** Bad 'nstepbio'; must be 1 or even for leapfrog scheme ***"
      endif

      do i=1,ntracer
        j=itracer(i)
        if (j/=0) tracer_mid(1:km,j)=tracer(1:km,j)
      enddo

    else

!     Set time-stepping parameters for leapfrog scheme

      dtstd=timestep*2
      nstepbio_lf=nstepbio

    endif

    dtbio=dtstd/nstepbio_lf

  else

!   Set time-stepping parameters for forward Euler 

    dtstd=timestep
    dtbio=dtstd/nstepbio

  endif 

! Initialize the general tendency for all activated tracers

  do i=1,ntracer
    j=itracer(i)
    if (j/=0) ddt(1:km,j)=0.0
  enddo

! Add advection tendencies

  if (advscheme/=0) then
    if (advscheme_ctflag .and. leapfrog) then  
      if (ecomodel/=0) call modelgetbiotrans(tracer_mid,wbio)
      call advection(advscheme,km,dz,dtstd,f_w,wbio,tracer_mid,ddt,ddth)   
    else
      if (ecomodel/=0) call modelgetbiotrans(tracer,wbio)
      call advection(advscheme,km,dz,dtstd,f_w,wbio,tracer,ddt,ddth)   
    endif
  endif

! Add perturbation tendencies 

  if (pertactive) then

    if (dtpert==0.0) then
      dtpert=-timestep   ! Negative value indicates new autoregressive process 
    else if (leapfrog .and. dtpert<0.0) then
      dtpert=timestep/2  ! Change from initial Euler time step to leapfrog
                         ! reduces forcing interval to half a base time step
    else
      dtpert=timestep 
    endif

    if (leapfrog) then
      call perturbation(km,dtstd,dtpert,f_pert,f_pertsd,tracer_mid,ddt,ddtp)
    else
      call perturbation(km,dtstd,dtpert,f_pert,f_pertsd,tracer,ddt,ddtp)
    endif

  endif 

! Add relaxation tendencies
! Always use 'before' tracer values for stability

  if (relaxactive) & 
    call relaxation(km,zlevbot,dtstd,f_mld,f_rlx,f_ref0,f_pertsd,zeuphotic, &
                    tracer,ddt,ddtr,relaxkmin,relaxkmax,relaxratefrac)

! Add source minus sink tendencies
! Time-splitting scheme is used if 'nstepbio' is greater than 1

  if (ecomodel/=0) then 
    if (leapfrog) then
      call ecosystem(nstepbio_lf,dtbio,tofy,yearlen,f_sol,f_solav, &
                     f_mld,f_sss,f_pco2atm,f_dust,f_fedust,f_temp, &
                     tracer,tracer_mid,ddt,ratio)
    else
      call ecosystem(nstepbio,dtbio,tofy,yearlen,f_sol,f_solav, &
                     f_mld,f_sss,f_pco2atm,f_dust,f_fedust,f_temp, &
                     tracer,tracer,ddt,ratio)
    endif
  endif

! Do diffusion (if any); calc. new model state including application of
! previously determined tendencies 
 
  if (leapfrog) then
    if (initialstep) then
      call diffusion(diffscheme,km,dz,dzw,dtstd,f_vdc,ddt,tracer,tracer_mid)
    else
      call diffusion(diffscheme,km,dz,dzw,dtstd,f_vdc,ddt,tracer,tracer_new)
    endif
  else
    call diffusion(diffscheme,km,dz,dzw,dtstd,f_vdc,ddt,tracer,tracer)
  endif

! Apply boundary layer mixing to model state

  if (mixscheme/=0) then
    if (leapfrog) then
      if (initialstep) then
        call mixing(mixscheme,kmixed,dz,dzmixed,tracer_mid)
      else
        call mixing(mixscheme,kmixed,dz,dzmixed,tracer_new)
      endif
    else
      call mixing(mixscheme,kmixed,dz,dzmixed,tracer)
    endif
  endif

! Apply Robert-Asselin time filter to output state & advance 'now' state for 
! next time step. 

  if (leapfrog .and. (.not.initialstep)) then

    do i=1,ntracer
      j=itracer(i)
      if (j/=0) then
        tracer(1:km,j)=pnu2m*tracer_mid(1:km,j)+ &
                       pnu*(tracer(1:km,j)+tracer_new(1:km,j))
        tracer_mid(1:km,j)=tracer_new(1:km,j)
      endif
    enddo

  endif

! Override tracers with prescribed fields where applicable 

  if (relaxactive) then
    if (leapfrog .and. initialstep) then
      call override(km,relaxkmin,relaxkmax,dtstd,f_rlx,f_ref1,tracer_mid,ddtr)
    else
      call override(km,relaxkmin,relaxkmax,dtstd,f_rlx,f_ref1,tracer,ddtr)
    endif
  endif

! Data assimilation analysis & update. 
! Done at the end of each assimilation time period spanning 'assimstep' base
! time steps. The 'assimnext' switch is set at the time step before to request
! input data at the next step. 
! The model determines the length of the assimilation time period from that of 
! the biological time step ('dtbio') by counting 'modeltrend' calls.  
! 'dtbio' is expected to be constant for the duration of an assimilation period.
! In leapfrog scheme, DA increments are applied at 2 time levels.

  if (assimscheme > 0 .and. (.not.initialstep)) then

    assimstepcount=assimstepcount+1
    if (assimstepcount>assimstep) assimstepcount=1

    if (assimstepcount==assimstep) then 
      call assimilation(dtbio,f_mld,f_cchl,f_chl,f_pgrow,f_ploss, &
                        tracer,tracer_mid,ratio,assimupd)
    else 
      assimupd=0
    endif

    if ((assimstep==1 .and. assimstepcount==1) .or.  &
        assimstepcount==assimstep-1) then
      assimnext=1
    else
      assimnext=0
    endif

  endif

! Reset leapfrog initial step switch

  initialstep=.FALSE.

  end subroutine

!=======================================================================

  subroutine modelgetbiotrans(tracer,w)
 
  use modconfig, only: i_det

  implicit none

! Input argument

  real(kind_r_ext) tracer(:,:)   ! Tracer values

! Output argument

  real(kind_r) w(:,:)            ! Tracer-specific vel. (m/d, up)
!-----------------------------------------------------------------------
! DEVNOTES: 
!  - Tracer values not used by any of the current models.
!  - Some models could require light field as input here which would 
!    neccessitate separating optics model from photosynthesis.

  real(kind_r) wdet

  select case(ecomodel)
  case(og99npzdmodel)
    call og99npzd_getbiotrans(wdet) 
  case(hadoccmodel)
    call hadocc_getbiotrans(wdet) 
  case(medusamodel)
    call medusa_getbiotrans(wdet)
  end select

  w(1:km,i_det)=wdet 

  end subroutine

!=======================================================================

  subroutine ecosystem(nstep,dt,tofy,yearlen,f_sol,f_solav,f_mld, &
                       f_sss,f_pco2atm,f_dust,f_fedust,f_temp, &
                       tracer,tracer_now,ddt,ratio)

  use photos, only: photosynthesis

  implicit none

! Input arguments

  integer nstep                     ! Number of time steps to average over
  real(kind_r) dt                   ! Length of time step (d) 
  real(kind_r_ext) tofy             ! Time of year (d)
  integer yearlen                   ! Year length (d)
  real(kind_r_ext) f_sol            ! Surface solar irradiance (W/m^2)
  real(kind_r_ext) f_solav          ! Daily mean surface irradiance (W/m^2)
  real(kind_r_ext) f_mld            ! Upper mixed layer depth (m)
  real(kind_r_ext) f_sss            ! Sea surface salinity
  real(kind_r_ext) f_pco2atm        ! Atmospheric pCO2 (uatm)
  real(kind_r_ext) f_dust           ! Aeolian dust deposition (mg/m^2/d) 
  real(kind_r_ext) f_fedust         ! Aeolian soluble iron (mmolFe/m^2/d) 
  real(kind_r_ext) f_temp(:)        ! Temperature (deg C)
  real(kind_r_ext) tracer(:,:)      ! Initial tracer values for 
                                    !   standard time step
  real(kind_r_ext) tracer_now(:,:)  ! Tracer values for determining 
                                    !   now tendencies 

! In/out argument

  real(kind_r_ext) ddt(:,:)         ! Tracer tendencies (tracer units /d)
                                    !   transport tendencies in 
                                    !   transport + source minus sink out 
  real(kind_r_ext) ratio(:,:)       ! Composition ratios 

!-----------------------------------------------------------------------

  real(kind_r_ext) f_solar     ! Surface solar irradiance for 
                               ! MarMOT photosynthesis calc. (W/m^2)
  real(kind_r) tofyfrac        ! Time of year (fraction)
  real(kind_r) par0            ! Surface PAR (E/m^2/d)
  real(kind_r) pigment(km)     ! Pigment concentration (mg/m^3) 
  real(kind_r) alpha0(nphy)    ! Initial slope of P-E curve for scalar PAR
                               !  at depth 0+ (gC/gChl/(E/m^2) OR /(E/m^2))
  logical chlflag(km,nphy)     ! Switch for presence of chlorophyll 
  real(kind_r) cchl(km,nphy)   ! C:Chl ratio (gC/gChl)
  real(kind_r) ddttrans(km,ntracer)
                               ! Tracer tendencies due to transport only
                               !   (tracer units /d)  

                                        ! MarMOT diagnostics arrays
  real(kind_r_ext) diagprof_tmp(km,ndp) !  - working copy of profiles 
  real(kind_r_ext) diagscal_tmp(nds)    !  - working copy of scalars 

  integer istep,midstep,midstep2,i,j,k
  real(kind_r_ext) x

  tofyfrac=tofy/yearlen

! Determine photosynthetic rate(s) using 'now' state 

! If there is no external photosynthesis model then zero the relevant 
! ecosystem model input data in case they are used in the absence of an 
! internal photosyntheis model. 

  select case(photosmodel)
  case(0) 
    alpha(:,:)=0.0
    vp(:,:)=0.0 
    photav(:,:)=0.0
    rphotaeav(:,:)=0.0
    parfraczbot(:)=0.0
    zeuphotic=0.0
  case(1)
    f_solar=f_solav
  case(2)
    f_solar=f_sol
  case(3)
    f_solar=f_solav
  end select

  if (photosmodel>0) then

    call modelgetphotin(f_solar,f_temp,tracer_now,ratio,par0, &
                        pigment,alpha0,chlflag,cchl,limdin,vp)

    call photosynthesis(photosmodel,attenmodel,attenpathopt, &
           achlmodel,latitude,tofyfrac,par0,attenwater, &
           attenpig,alpha0,km,kmphotic,dz,pigment,vp,chlflag,cchl, &
           kdpar,parfraczbot,parzbot,parfracz,parz,alpha,photz, &
           photav,rphotaeav,zeuphotic,daylen)

  endif

! Get 'now' tendencies

  if (nstep==1) then

!   Determine 'now' tendencies from 'now' state.
!   Source-minus-sink tendencies are added to transport. 

    call modeltrend(dt,tofy,yearlen,f_solar,f_sol,f_solav,f_mld, &
                    f_sss,f_pco2atm,f_dust,f_fedust,f_temp, &
                    tracer_now,ratio,ddt)

  else

!   Advance the model state by 'nstep' forward Euler sub-steps and
!   determine mean tendencies from the differences between initial
!   and final states. 
!   Tendencies for each sub-step are initialized from transport tendencies
!   and updated with source-minus-sink.
!   Diagnostics are saved for the middle of the standard time step. For an 
!   even number of sub-steps this is an interpolated value. 

    do i=1,ntracer
      j=itracer(i)
      if (j/=0) then
        tracer_new(1:km,j)=tracer(1:km,j)
        ddttrans(1:km,i)=ddt(1:km,j)
      endif
    enddo

    midstep=nstep/2
    if (midstep*2 == nstep) then
      midstep2=midstep+1
    else
      midstep=midstep+1
      midstep2=0
    endif

    do istep=1,nstep

      call modeltrend(dt,tofy,yearlen,f_solar,f_sol,f_solav,f_mld, &
                      f_sss,f_pco2atm,f_dust,f_fedust,f_temp, &
                      tracer_new,ratio,ddt)

!     Save diagnostics at mid point of standard time step

      if (istep==midstep) then 
        call getdiagnostics_biostep(diagprof_tmid,diagscal_tmid)
      else if (istep==midstep2) then
        call getdiagnostics_biostep(diagprof_tmp,diagscal_tmp)
        do i=1,ndp
          do k=1,km
            x=diagprof_tmid(k,i) 
            if (x /= objf_nodata) diagprof_tmid(k,i)=0.5*(x+diagprof_tmp(k,i))
          enddo
        enddo
        do i=1,nds
          x=diagscal_tmid(i)
          if (x /= objf_nodata) diagscal_tmid(i)=0.5*(x+diagscal_tmp(i))
        enddo
      endif

!     Update tracers and set tendencies for next sub-step or standard step 

      do i=1,ntracer
        j=itracer(i)
        if (j/=0) then
            tracer_new(1:km,j)=tracer_new(1:km,j)+dt*ddt(1:km,j)
            if (istep<nstep) then
              ddt(1:km,j)=ddttrans(1:km,i)
            else
              ddt(1:km,j)=(tracer_new(1:km,j)-tracer(1:km,j))/(nstep*dt)
            endif
        endif
      enddo

    enddo

  endif

  end subroutine

!=======================================================================

  subroutine modelgetphotin(f_solar,f_temp,tracer,ratio,par0,pigment, &
                                alpha0,chlflag,cchl,limdin_phot,vp_phot)
   
  implicit none

! Input arguments

! Input arguments

  real(kind_r_ext) f_solar     ! Surface solar irradiance (W/m^2)
  real(kind_r_ext) f_temp(:)   ! Temperature (deg C)
  real(kind_r_ext) tracer(:,:) ! Tracer values 
  real(kind_r_ext) ratio(:,:)  ! Composition ratios

! Output argument

  real(kind_r) par0            ! Surface PAR (E/m^2/d)
  real(kind_r) pigment(km)     ! Pigment concentration (mg/m^3)
  real(kind_r) alpha0(nphy)    ! Initial slope of P-E curve for scalar PAR
                               !  at depth 0+ (gC/gChl/(E/m^2) OR /(E/m^2))
  logical chlflag(km,nphy)     ! Switch for presence of chlorophyll (mg/m^3)
  real(kind_r) cchl(km,nphy)   ! C:Chl ratio (gC/gChl)
  real(kind_r) limdin_phot(:,:)! DIN limitation factor 
  real(kind_r) vp_phot(:,:)    ! Phytoplankton max. growth rate (/d)

!-----------------------------------------------------------------------

  integer k

  select case(ecomodel)

  case(og99npzdmodel)
    call og99npzd_getphotin(f_solar,f_temp,tracer,par0, &
                            pigment,alpha0(1),chlflag(:,1),vp(:,1))
    do k=1,km
      cchl(k,1)=1.0_kind_r  ! dummy conversion: alpha0 is carbon-specific
    enddo

  case(hadoccmodel)
    call hadocc_getphotin(f_solar,f_temp,tracer,ratio,par0, &
           pigment,alpha0(1),chlflag(:,1),cchl(:,1),limdin(:,1),vp(:,1))

  case(medusamodel)
    call medusa_getphotin(f_solar,f_temp,tracer,par0,pigment, &
                          alpha0,chlflag,cchl,vp)

  end select

  end subroutine

!=======================================================================

  subroutine modeltrend(dt,tofy,yearlen,f_solphot,f_sol,f_solav, &
                        f_mld,f_sss,f_pco2atm,f_dust,f_fedust, &
                        f_temp,tracer,ratio,ddt)

  implicit none

! Input arguments

! Input arguments

  real(kind_r) dt                   ! Length of biological time step (d) 
  real(kind_r_ext) tofy             ! Time of year (d)
  integer yearlen                   ! Year length (d)
  real(kind_r_ext) f_solphot        ! Surface irradiance used in MarMOT 
                                    !   photosynthesis (W/m^2)
  real(kind_r_ext) f_sol            ! Surface solar irradiance now (W/m^2)
  real(kind_r_ext) f_solav          ! Daily mean surface irradiance (W/m^2)
  real(kind_r_ext) f_mld            ! Upper mixed layer depth (m)
  real(kind_r_ext) f_sss            ! Sea surface salinity
  real(kind_r_ext) f_pco2atm        ! Atmospheric pCO2 (uatm)
  real(kind_r_ext) f_dust           ! Aeolian dust deposition (mg/m^2/d)
  real(kind_r_ext) f_fedust         ! Aeolian soluble iron (mmolFe/m^2/d)
  real(kind_r_ext) f_temp(:)        ! Temperature (deg C)
  real(kind_r_ext) tracer(:,:)      ! Tracer values 

! In/out argument

  real(kind_r_ext) ratio(:,:)       ! Composition ratios
  real(kind_r_ext) ddt(:,:)         ! Tracer tendencies (tracer units /d)
                                    !   transport tendencies in
                                    !   transport + source-minus-sink out
!-----------------------------------------------------------------------
! The model tendencies are added to the input tendencies.
! If assimilation is active, the model is also responsible for updating
! a biological dynamics record with data used in the assimilation scheme.
! This routine is intended to be called once only at each biological 
! time step.   
!-----------------------------------------------------------------------

  real(kind_r_ext) f_solar   

  select case(ecomodel)

  case(og99npzdmodel)

    call og99npzd_biotrend(f_temp,photav(:,1),tracer,ddt)

  case(hadoccmodel)

    if (photosmodel==0) then
      f_solar=f_solav
    else
      f_solar=f_solphot
    endif

    call hadocc_biotrend(dt,tofy,yearlen,f_solar,f_mld,f_temp,f_temp(1), &
           f_sss,f_pco2atm,kmphotic,photav(:,1),alpha(:,1),vp(:,1), &
           parfraczbot,tracer,ratio,ddt,zeuphotic)

  case(medusamodel)

    call medusa_biotrend(f_sol,f_temp,f_dust,f_fedust,photav, &
                         rphotaeav,tracer,ddt,zeuphotic)

  end select

  end subroutine

!=======================================================================

  subroutine assimilation(dt,f_mld,f_cchl,f_chl,f_pgrow,f_ploss, &
                          tracer,tracer_nxt,ratio,assimupd)

  implicit none

! Input arguments

  real(kind_r) dt                  ! Length of biological time step (d) 
  real(kind_r_ext) f_mld           ! Upper mixed layer depth (m)
  real(kind_r_ext) f_cchl          ! Surface C:Chl (gC/gChl)
  real(kind_r_ext) f_chl           ! Surface chlorophyll (mg/m3)
  real(kind_r_ext) f_pgrow         ! Mixed layer phytoplankton specific
                                   ! growth rate (\d) 
  real(kind_r_ext) f_ploss         ! Mixed layer phytoplankton specific
                                   ! loss rate (\d) 

! In/out argument

  real(kind_r_ext) tracer(:,:)     ! Tracer values for background state 

! Output argument

  real(kind_r_ext) tracer_nxt(:,:) ! Tracer values for update only
                                   !   These are the values one standard 
                                   !   time step ahead of the output 
                                   !   values in the leapfrog scheme 
  real(kind_r_ext) ratio(:,:)      ! Composition ratios
  integer(kind_i_ext) assimupd     ! Flag set on analysis update

!-----------------------------------------------------------------------

  assimupd=0

  select case(ecomodel)
  case(hadoccmodel)     

    call hadocc_ocda_analysis(dt,f_mld,f_cchl,f_chl, &
                              f_pgrow,f_ploss,tracer,ratio)

    if (leapfrog) then
      call hadocc_ocda_applyinc(tracer)
      call hadocc_ocda_applyinc(tracer_nxt)
    else
      call hadocc_ocda_applyinc(tracer)
    endif
    assimupd=1

  end select
 
  end subroutine

!=======================================================================

#ifdef no_iso_c_binding

  subroutine getdiagnostics(diagprof,diagscal)
      
#else

  subroutine getdiagnostics(diagprof,diagscal) &
             bind(C,name="FORTRAN_getdiagnostics")

#endif

  use modconfig, only: i_mixed,i_kdpar,i_parfracztop,i_parztop, &
                       i_parfracz,i_parz, &
                       i_limdin,i_pmax,i_alpha,i_pz,i_pk, &
                       i_photmaxndia,i_pmaxndia,i_alphandia, &
                       i_pzndia,i_pkndia,i_photndia, &
                       i_photmaxdiat,i_pmaxdiat,i_alphadiat, &
                       i_pzdiat,i_pkdiat,i_photdiat, &
                       i_rlxfrac,i_zeuph,i_daylen,i_det,i_wdet

  use trrelax, only: relaxactive,relaxctrlfact

  implicit none

! Arguments (output) - diagnostics accessed by MarMOT indices

  real(kind_r_ext) diagprof(nk,ndp)        ! Profile diagnostics
  real(kind_r_ext) diagscal(nds)           ! Scalar diagnostics 

!-----------------------------------------------------------------------

  real(kind_r_ext) x
  integer i,j,k

! Detrital sinking velocity 
! (to be overwritten by ecosystem model if sinking is handled internally)

  if (detsinkflag) diagprof(1:km,i_wdet)=wbio(1:km,i_det)

! Ecosystem model diagnostics

  if (nstepbio==1) then 
    call getdiagnostics_biostep(diagprof,diagscal)
  else
    do i=1,ndp
      do k=1,km
        x=diagprof_tmid(k,i)
        if (x /= objf_nodata) diagprof(1:km,i)=x
      enddo
    enddo
    do i=1,nds
      x=diagscal_tmid(i)
      if (x /= objf_nodata) diagscal(i)=x
    enddo
  endif

! Set detrital sinking to zero at bottom of water column
        
  if (detsinkflag) diagprof(km,i_wdet)=0.0 

! Fraction of level in mixed layer 

  if (mixscheme/=0) then
    diagprof(1:kmixed,i_mixed)=1.0_kind_r_ext
    k=kmixed+1
    if (k<=km .and. mixscheme==2) then
      diagprof(k,i_mixed)=dzmixed/dz(k)
      k=k+1
    endif
    diagprof(k:km,i_mixed)=0.0
  endif

! Diagnostics from external photosynthesis sub-model

! N.B. Values calculated in the model 'getphotin' routines and in the 
! photosyntheis sub-model are handled here and should not be assigned
! in model 'getdiag' routines except when a model is using its own
! internal photosynthesis routine 

  if (ecomodel/=0 .and. photosmodel>0) then

    do k=1,km

      if (parfraczbot(k-1)>0.0) then
        diagprof(k,i_kdpar)=kdpar(k)
      endif
      diagprof(k,i_parfracztop)=parfraczbot(k-1)
      diagprof(k,i_parztop)=parzbot(k-1)
      diagprof(k,i_parfracz)=parfracz(k)
      diagprof(k,i_parz)=parz(k)

      select case(ecomodel)

      case(og99npzdmodel)
        diagprof(k,i_pmax)=vp(k,1)   
        if (parfraczbot(k-1)>0.0) diagprof(k,i_alpha)=alpha(k,1)
        if (photosmodel==2) diagprof(k,i_pz)=photz(k,1)
        diagprof(k,i_pk)=photav(k,1)

      case(hadoccmodel)
        diagprof(k,i_limdin)=limdin(k,1)    
        diagprof(k,i_pmax)=vp(k,1)
        if (parfraczbot(k-1)>0.) diagprof(k,i_alpha)=alpha(k,1)
        if (photosmodel==2) diagprof(k,i_pz)=photz(k,1)
        diagprof(k,i_pk)=photav(k,1)

      case(medusamodel)
        diagprof(k,i_photmaxndia)=vp(k,1)
        diagprof(k,i_pmaxndia)=vp(k,1)
        if (parfraczbot(k-1)>0.0) diagprof(k,i_alphandia)=alpha(k,1)
        if (photosmodel==2) diagprof(k,i_pzndia)=photz(k,1)
        diagprof(k,i_pkndia)=photav(k,1)
        diagprof(k,i_photmaxdiat)=vp(k,2)
        diagprof(k,i_pmaxdiat)=vp(k,2)
        if (parfraczbot(k-1)>0.0) diagprof(k,i_alphadiat)=alpha(k,2)
        if (photosmodel==2) diagprof(k,i_pzdiat)=photz(k,2)
        diagprof(k,i_pkdiat)=photav(k,2)

      end select

    enddo

    diagprof(km+1,i_parfracztop)=parfraczbot(km)
    diagprof(km+1,i_parztop)=parzbot(km)

  endif

! Fluxes into and out of the water column

  do i=1,ntracer
    j=itracer(i)
    if (j/=0) then
      diagprof(1:km,dpoff_ddth+j)=ddth(1:km,j)
      diagprof(1:km,dpoff_ddtp+j)=ddtp(1:km,j)
      diagprof(1:km,dpoff_ddtr+j)=ddtr(1:km,j)
    endif
  enddo

! Scalar diagnostics: relaxation scaling factor, daylength, 
!                     euphotic zone depth

  if (relaxactive .and. relaxctrlfact>0) diagscal(i_rlxfrac)=relaxratefrac 

  if (ecomodel/=0) then
    if (photosmodel==1 .or. photosmodel==3) diagscal(i_daylen)=daylen
    diagscal(i_zeuph)=zeuphotic
  endif
          
  end subroutine
    
!=======================================================================
    
  subroutine getdiagnostics_biostep(diagprof,diagscal)
      
  use modconfig, only: i_wdet,i_insinkn,i_insinkc,i_insinkfe
    
  implicit none
      
! Arguments (output) - diagnostics accessed by MarMOT indices
      
  real(kind_r_ext) diagprof(:,:)         ! Profile diagnostics
  real(kind_r_ext) diagscal(:)           ! Scalar diagnostics

!-----------------------------------------------------------------------

  select case(ecomodel)
  case(og99npzdmodel) 
    call og99npzd_getdiag(diagprof)
  case(hadoccmodel)
    call hadocc_getdiag(diagprof,diagscal)
  case(medusamodel)
    call medusa_getdiag(diagprof,diagscal)
  end select
       
! MarMOT sinking particle flux diagnostics require both fast and
! slow sinking data for N, C & Fe.
! For models that only produce EITHER fast transfer rates OR slow sinking
! velocities, the missing data must be replaced by zeros to ensure
! that all possible diagnostics are included in the output.

  select case(ecomodel)
  case(og99npzdmodel)
    diagprof(1:km,i_insinkn)=0.0
    diagprof(1:km,i_insinkc)=0.0
    diagprof(1:km,i_insinkfe)=0.0
  case(hadoccmodel)
    diagprof(1:km,i_insinkn)=0.0
    diagprof(1:km,i_insinkfe)=0.0
  end select

  end subroutine

!=======================================================================

#ifdef no_iso_c_binding

  subroutine getassimreport(outprof,outscal)

#else

  subroutine getassimreport(outprof,outscal) &
             bind(C,name="FORTRAN_getassimreport")

#endif

  use modconfig, only: ndp_ocda,nds_ocda 

  implicit none

! Arguments (output)

  real(kind_r_ext) outprof(nk,ndp_ocda)        ! Profile output 
  real(kind_r_ext) outscal(nds_ocda)           ! Scalar output 

!-----------------------------------------------------------------------

  if (assimscheme==1) then

    select case(ecomodel)
    case(hadoccmodel) 
      call hadocc_ocda_getdiag(outprof,outscal)
    end select

  endif

  end subroutine

!=======================================================================

  end module modsock

