!**************************************************************************!
! SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1 !
!**************************************************************************!
!**************************************************************************!
!                                                                          ! 
! SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013                     ! 
!                                                                          !
! Copyright Natural Environment Research Council, UK                       !
!                                                                          !
! Author: John Hemmings, National Oceanography Centre                      !
! e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013)                         !
! Alternative e-mail for author: marmot@wessexenv.mail1.co.uk              !
!                                                                          !
! This software is governed by the CeCILL  license under French law and    !
! abiding by the rules of distribution of free software.  You can  use,    ! 
! modify and/ or redistribute the software under the terms of the CeCILL   !
! license as circulated by CEA, CNRS and INRIA at the following URL        !
! "http://www.cecill.info".                                                ! 
!                                                                          !
! As a counterpart to the access to the source code and  rights to copy,   !
! modify and redistribute granted by the license, users are provided only  !
! with a limited warranty  and the software's author,  the holder of the   !
! economic rights,  and the successive licensors  have only  limited       !
! liability.                                                               ! 
!                                                                          !
! In this respect, the user's attention is drawn to the risks associated   !
! with loading,  using,  modifying and/or developing or reproducing the    !
! software by the user in light of its specific status of free software,   !
! that may mean  that it is complicated to manipulate,  and  that  also    !
! therefore means  that it is reserved for developers  and  experienced    !
! professionals having in-depth computer knowledge. Users are therefore    !
! encouraged to load and test the software's suitability as regards their  !
! requirements in conditions enabling the security of their systems and/or ! 
! data to be ensured and,  more generally, to use and operate it in the    ! 
! same conditions as regards security.                                     ! 
!                                                                          ! 
! The fact that you are presently reading this means that you have had     !
! knowledge of the CeCILL license and that you accept its terms.           !
!                                                                          !
!**************************************************************************!
!=======================================================================
!     MODULE photos 
!     John Hemmings - 12 Nov 2013 
!
!     MarMOT optics & photosynthesis sub-models 
!
!     Subroutines:
!
!      - photsetup
!
!         - a93kdsetup_original
!                           set up routine for Anderson 1993 PAR 
!                           attenuation model with original layers
!                           (layer boundaries at 5m, 23m) 
!                   
!         - a93kdsetup_hadocc      
!                           set up routine for Anderson 1993 PAR 
!                           attenuation model with HadOCC layers
!                           (layer boundaries at 10m, 20m)
!
!         - a93kdsetup      common set up code for Anderson 1993 PAR 
!                           attenuation model 
!
!         - a93achlsetup    set up routine for Anderson 1993 chlorophyll-
!                           specfic absorption coefficient model
!
!      - photosynthesis
!
!         - a93optics       Anderson 1993 model for the attenuation of 
!                           downwelling PAR due to water and pigment
!                           concentration 
! 
!         - og99patheffect  adjusts attenuation coefficient (following
!                           Oschlies and Garcon, 1999) to take into account 
!                           sun's angle on the assumption that direct 
!                           path effects dominate (tends to bias 'kd' high)
!                           N.B. noon angle is used so path length are
!                           shorter than true direct paths for other
!                           times of day (tends to bias 'kd' low)
!
!         - a93spectralachl Anderson 1993 model of variation in chlorophyll-
!                           specific absorption coefficient with depth
!                           variation in PAR spectrum due to effect of 
!                           water and pigment concentration  
!
!         - photos_og99ep   Oschlies and Garcon 1999 sub-model 
!                           based on Evans & Parslow (1985) analytical 
!                           integral with triangular diel light cycle 
!                           - requires daily mean surface PAR 
!
!           - func_ep85     Evans & Parslow 1985 integral function
!
!         - photos_smith    simple point-in-time photosynthesis model 
!                           based on Smith function (Smith, 1936) 
!                           - also calculates ratio of photosynthesis 
!                             to alpha*PAR for parameterization of light 
!                             adaptation 
!
!         - photos_platt90  Platt et al 1990 photosyntheis model as 
!                           applied by Anderson 1993 
!                           - requires daily mean surface PAR
!
!           - pfunc         Platt et al 1990 approximation to analytical 
!                           function for normalized depth-integrated
!                           photosynthesis (extended by Evans & Anderson)
!
!      References:
!
!      Anderson, T.R. 1993. A spectrally averaged model of light 
!      penetration and photosynthesis. Limnol.  Oceanogr. 38, 1403-1419. 
!      Evans, G.T. and Parslow, J.S. 1985. A model of annual plankton 
!      cycles. Biol. Oceanogr. 3, 327-347.
!      Oschlies, A. and Garcon, V. 1999. An eddy-permitting coupled 
!      physical-biological model of the North Atlantic. 1. Sensitivity 
!      to advection numerics and mixed layer physics. Global Biogeochem. 
!      Cy. 13, 135-160.
!      Platt, T., Sathyendranath, S. and Ravindran, P. 1990. Primary 
!      production by phytoplankton: Analytic solutions for daily rates per 
!      unit area of water surface. Proc. R. Soc. Lond. Ser. B 241, 101-111.
!      Smith, E.L. 1936. Photosynthesis in relation to light and carbon 
!      dioxide. Proc. Natl. Acad. Sci. U.S.A. 22, 504–511.
!
!=======================================================================

      module photos 

      use modconfig, only: kind_r_ext,kind_r,objf_nodata,nk

      real(kind_r),parameter :: par_threshold=0.00001
                                ! Threshold PAR value (E/m^2/d) below which we 
                                ! assume no photosynthesis

      real(kind_r) zkdlayer(2)  ! Conceptual layer boundaries for
                                ! Anderson 93 PAR attenuation model 

      real(kind_r) kdb0(nk),kdb1(nk),kdb2(nk), &
                   kdb3(nk),kdb4(nk),kdb5(nk)
                                ! Depth level coefficients for Anderson 93 
                                ! PAR attenuation model         

      real(kind_r) co_dahash(3,nk) 
                                ! Depth level coefficients for Anderson 93 
                                ! chlorophyll absorption model

      contains

!=======================================================================

      subroutine photsetup(km,dz,zlevbot,kdmod,achlmod,kmphotic)

      implicit none

!     Input arguments

      integer km               ! Number of levels
      real(kind_r) dz(:)       ! Level height (m) 
      real(kind_r) zlevbot(:)  ! Depth at bottom of level (m)
      integer kdmod            ! Light attenuation model 
                               !  1 = linear vs. pigment  
                               !  2 = A93 original layers (0-5m, 5-23m, 23m+) 
                               !  3 = A93 HadOCC layers (0-10m, 10-20m, 20m+) 
      integer achlmod          ! Light absorption model
                               !  0 = none 
                               !  1 = Anderson 93 

!     Output arguments 

      integer kmphotic         ! Maximum depth level for photosynthesis 

!-----------------------------------------------------------------------

      integer k

      kmphotic=0
      k=1
      do while (kmphotic==0 .and. k<=km)
        if (zlevbot(k) >= 200.0_kind_r) kmphotic=k    
        k=k+1
      enddo
      if (kmphotic==0) kmphotic=km

      if (kdmod==2) then
        call a93kdsetup_original(km,dz)
      else if (kdmod==3) then
        call a93kdsetup_hadocc(km,dz)
      endif

      if (achlmod==1) call a93achlsetup(km,dz)

      end subroutine

!=======================================================================
        
      subroutine a93kdsetup_original(km,dz)
        
      implicit none
        
      integer km
      real(kind_r) dz(:)

!-----------------------------------------------------------------------

      real(kind_r) zlayerbot(2)
      real(kind_r) b0(3),b1(3),b2(3),b3(3),b4(3),b5(3)
      
                    ! Coeffs. in the polynomial expression of the light
                    ! extinction coeff. as a function of pigment
                    ! concentration (Anderson 1993).
                    ! Each coefficent takes one of three values, 
                    ! depending on the conceptual layer of the light 
                    ! model; layer boundaries are at 5 and 23 m.
      
      data b0 / 0.13096 , 0.041025 , 0.021517 / 
      data b1 / 0.030969 , 0.036211 , 0.050150 /
      data b2 / 0.042644 , 0.062297 , 0.058900 /
      data b3 / -0.013738 , -0.030098 , -0.040539 /
      data b4 / 0.0024617 , 0.0062597 , 0.0087586 /
      data b5 / -0.00018059 , -0.00051944 , -0.00049476 /
        
      data zlayerbot / 5 , 23 /
        
      call a93kdsetup(zlayerbot,b0,b1,b2,b3,b4,b5,km,dz)
       
      end subroutine

!=======================================================================

      subroutine a93kdsetup_hadocc(km,dz)

      implicit none

      integer km
      real(kind_r) dz(:)

!-----------------------------------------------------------------------

      real(kind_r) zlayerbot(2)
      real(kind_r) b0(3),b1(3),b2(3),b3(3),b4(3),b5(3)

                    ! Coeffs. in the polynomial expression of the light
                    ! extinction coeff. as a function of pigment
                    ! concentration (Totterdell, pers. comm., 2004). 
                    ! These are alternative values determined by Tom 
                    ! Anderson for layers matching levels in the Met  
                    ! Office HadOCC model.
                    ! Each coefficent takes one of three values, 
                    ! depending on the conceptual layer of the light 
                    ! model; layer boundaries are at 10 and 20 m.

      data b0 / 0.095934 , 0.026590 , 0.015464 /
      data b1 / 0.039307 , 0.016301 , 0.14886  /
      data b2 / 0.051891 , 0.073944 ,-0.15711  /
      data b3 /-0.020760 ,-0.038958 , 0.15065  /
      data b4 / 0.0043139  , 0.0075507  ,-0.055830  /
      data b5 /-0.00035055 ,-0.00054532 , 0.0075811 /

      data zlayerbot / 10 , 20 /

      call a93kdsetup(zlayerbot,b0,b1,b2,b3,b4,b5,km,dz)
 
      end subroutine

!=======================================================================

      subroutine a93kdsetup(zlayerbot,b0,b1,b2,b3,b4,b5,km,dz)

      implicit none

      real(kind_r) zlayerbot(:)
      real(kind_r) b0(:),b1(:),b2(:),b3(:),b4(:),b5(:)
      integer km
      real(kind_r) dz(:)

!-----------------------------------------------------------------------

      real(kind_r) ztop,zbot
      real(kind_r) w1,w2,w3
      integer k

      ztop=0.0
      zbot=dz(1)

      do k=1,km

        if (ztop < zlayerbot(2)) then

          if (ztop < zlayerbot(1)) then
            w1=(min(zlayerbot(1),zbot)-ztop)/dz(k)
          else
            w1=0.0
          endif

          if (ztop < zlayerbot(2) .and. zbot > zlayerbot(1)) then
            w2=(min(zlayerbot(2),zbot)-max(zlayerbot(1),ztop))/dz(k)
          else
            w2=0.0
          endif

          if (zbot > zlayerbot(2)) then
            w3=(zbot-max(zlayerbot(2),ztop))/dz(k)
          else
            w3=0.0
          endif

          kdb0(k)=w1*b0(1)+w2*b0(2)+w3*b0(3)
          kdb1(k)=w1*b1(1)+w2*b1(2)+w3*b1(3)
          kdb2(k)=w1*b2(1)+w2*b2(2)+w3*b2(3)
          kdb3(k)=w1*b3(1)+w2*b3(2)+w3*b3(3)
          kdb4(k)=w1*b4(1)+w2*b4(2)+w3*b4(3)
          kdb5(k)=w1*b5(1)+w2*b5(2)+w3*b5(3)

          ztop=zbot
          zbot=zbot+dz(k)

        else

          kdb0(k)=b0(3)
          kdb1(k)=b1(3)
          kdb2(k)=b2(3)
          kdb3(k)=b3(3)
          kdb4(k)=b4(3)
          kdb5(k)=b5(3)

        endif
  
      enddo

      end subroutine 

!=======================================================================
                            
      subroutine a93achlsetup(km,dz)

      implicit none
        
      integer km
      real(kind_r) dz(:)
      
!-----------------------------------------------------------------------
      
      real(kind_r) zp,f1,f2,f3,f1above,f2above,f3above
      integer k
      
!     Calculate coefficients for each depth level for use in the 
!     calculation of a# change over the level using the integral of da#/dz 
!     between level boundaries.
!     Each coefficient 'co_dahash' is the difference between function 
!     f1, f2 or f3 evaluated at the level bottom and level top. These are
!     the depth-dependent coefficients in the definite integral.     

      zp=1.0_kind_r
      f1=-zp
      f2=-2.0_kind_r*f1
      f3=-3.0_kind_r*f2

      do k=1,km

        f1above=f1
        f2above=f2
        f3above=f3

        zp=zp+dz(k)
        f1=zp*log(zp)-zp
        f2=zp*log(zp)**2-2.0_kind_r*f1
        f3=zp*log(zp)**3-3.0_kind_r*f2

        co_dahash(1,k)=f1-f1above
        co_dahash(2,k)=f2-f2above
        co_dahash(3,k)=f3-f3above

      enddo

      end subroutine

!=======================================================================

      subroutine photosynthesis(photmod,kdmod,kdpath,achlmod, &
        lat,tofy,par0,attenwater,attenpig,alphachl,km,kmphotic, &
        dz,pigment,vp,chlflag,cchl,kd,parfraczbot,parzbot,parfracz,parz, &
        alpha,photz,photav,rphotaeav,zeuphotic,daylen) 

      implicit none

!     Input arguments

      integer photmod          ! Phosynthesis model
      integer kdmod            ! Light attenuation model
                               !  0 = No pigment effect 
                               !  1 = linear vs. pigment
                               !  2 = A93 original layers (0-5m, 5-23m, 23m+)
                               !  3 = A93 HadOCC layers (0-10m, 10-20m, 20m+)
      integer kdpath           ! Light atten. model path-length adjustment 
                               !  0 = none
                               !  1 = Oschlies & Garcon 99 
      integer achlmod          ! Light absorption model 
                               !  0 = none  
                               !  1 = Anderson 93 
      real(kind_r) lat         ! Latitude
      real(kind_r) tofy        ! Time of year (years)
      real(kind_r) par0        ! Surface PAR (E/m^2/d)
      real(kind_r) attenwater  ! Attenuation due to water (/m)
      real(kind_r) attenpig    ! Atten. due to pigment (/m/(mg/m^3)) 
      real(kind_r) alphachl(:) ! Chl. specific initial slope of P-E curve for 
                               !   scalar PAR at depth 0+ (gC/gChl/(E/m^2))
      integer km               ! Number of levels
      integer kmphotic         ! Number of levels for photosyntheis
      real(kind_r) dz(:)       ! Level height (m) 
      real(kind_r) pigment(:)  ! Pigment conc. for PAR atten. (mg/m^3)
      real(kind_r) vp(:,:)     ! Phytoplankton maximum growth rate (/d)
      logical chlflag(:,:)     ! Switch for presence of chlorophyll 
      real(kind_r) cchl(:,:)   ! C:Chl ratio for alpha conversion (mgC/mgChl)

!     Notes

!      - 'lat' and 'tofy' are  not used if photmod=2 and kdpath=0 
!      - 'attenwater' and 'attenpig' are not used if 'kdmod' is 2 or 3 
!      - 'alphachl' can be passed a carbon specific alpha if 'cchl' is set to 1

!     Output arguments

      real(kind_r) kd(:)       ! Diffuse atten. coeff., downwelling PAR (/m)
      real(kind_r) parfraczbot(0:)
                               ! Fraction of surface PAR at bottom of level 
      real(kind_r) parzbot(0:) ! PAR at bottom of level (E/m^2/d) 
      real(kind_r) parfracz(:) ! Fraction of surface PAR at level mid-point
      real(kind_r) parz(:)     ! PAR at level mid-point (E/m^2/d)
      real(kind_r) alpha(:,:)  ! Initial slope of P-E curve, downwelling PAR
                               !   (/(E/m^2))
      real(kind_r) photz(:,:)  ! Carbon fixation at level mid-point (/d)
      real(kind_r) photav(:,:) ! Level mean carbon fixation (/d) 
      real(kind_r) rphotaeav(:,:)
                               ! Level mean photosynthesis:(alpha*par) ratio 
      real(kind_r) zeuphotic   ! Euphotic zone depth, 1% PAR (m) 
      real(kind_r) daylen      ! Day length (d) 

!-----------------------------------------------------------------------

      real(kind_r) rtpig(km)       ! Sqrt of pigment conc. (mg^(1/2)) 
      real(kind_r) ahash(kmphotic) ! Non-dimensional spectrally averaged 
                                   !   chl-specific absorption 
      real(kind_r) pi,dgamma,declin,lat_radians,x
      real(kind_r) sumkddz
      real(kind_r) zabove
      real(kind_r) alphaprime 
      integer i,k,nphy

!     Calculate day length
!     Declination is calculated from the day angle 'dgamma' using the 
!     approximation of Spencer J W (1971) "Fourier series representation of 
!     the position of the sun." Search 2(5), 172.

      if (photmod==1 .or. photmod==3 .or. kdpath==1) then

        pi=4.0_kind_r*atan(1.0_kind_r)
        dgamma=2.0_kind_r*pi*tofy

        declin=0.006918_kind_r &
               -0.399912_kind_r*cos(dgamma) &
               +0.070257_kind_r*sin(dgamma) &
               -0.006758_kind_r*cos(2.0_kind_r*dgamma) &
               +0.000907_kind_r*sin(2.0_kind_r*dgamma) &
               -0.002697_kind_r*cos(3.0_kind_r*dgamma) &
               +0.001480_kind_r*sin(3.0_kind_r*dgamma)

        lat_radians=(pi/180.0_kind_r)*lat

        x=min(1.0_kind_r,-tan(lat_radians)*tan(declin))
        x=max(-1.0_kind_r,x)
        daylen=acos(x)/pi

      endif


!     Determine attenuation coefficient profile

      if (kdmod > 1 .or. achlmod == 1) then
        do k=1,km
          rtpig(k)=min(sqrt(pigment(k)),3.873_kind_r)
        enddo
      endif 

      select case(kdmod)
      case(0)
        do k=1,km
          kd(k)=attenwater
        enddo
      case(1)
        do k=1,km
          kd(k)=attenwater+attenpig*pigment(k)
        enddo
      case(2)
        call a93optics(km,rtpig,kd)
      case(3)
        call a93optics(km,rtpig,kd)
      end select

      if (kdpath == 1) call og99patheffect(lat_radians,declin,km,kd)


!     Determine PAR field values and euphotic zone depth

      zeuphotic=-1
      zabove=0.0
      sumkddz=0.0
      parfraczbot(0)=1.0_kind_r
      parzbot(0)=par0

      do k=1,km
 
        if (k<=kmphotic .or. zeuphotic<0.) then

          parfracz(k)=exp(-sumkddz-0.5_kind_r*kd(k)*dz(k))
          sumkddz=sumkddz+kd(k)*dz(k)
          parfraczbot(k)=exp(-sumkddz)
          parz(k)=par0*parfracz(k)
          parzbot(k)=par0*parfraczbot(k)

          if (zeuphotic<0.0) then
            if (parfraczbot(k)<=0.01_kind_r) then
              zeuphotic=zabove+(log(parfraczbot(k-1))-log(0.01_kind_r))/kd(k)
            else if (k==km) then
              zeuphotic=zabove+dz(k)
            else
              zabove=zabove+dz(k)
            endif
          endif

        else

          parfracz(k)=0.0
          parfraczbot(k)=0.0
          parz(k)=0.0

        endif

      enddo

!     Determine P-E initial slope profile(s) and calculate photosynthesis

      nphy=size(alphachl)
      do i=1,nphy

        if (achlmod == 1) then
          alphaprime=2.602_kind_r*alphachl(i)
          call a93spectralachl(kmphotic,dz,rtpig,ahash)
          do k=1,kmphotic
            if (chlflag(k,i)) then
              alpha(k,i)=max(0.0,alphaprime*ahash(k)/cchl(k,i))
            else  
              alpha(k,i)=0.0
            endif
          enddo
        else
          do k=1,kmphotic
            if (chlflag(k,i)) then
              alpha(k,i)=alphachl(i)/cchl(k,i)
            else  
              alpha(k,i)=0.0
            endif
          enddo
        endif

        select case(photmod)

        case(1)
          call photos_og99ep(daylen,kmphotic,dz,parzbot,kd, &
                             alpha(:,i),vp(:,i),photav(:,i))
          photz(:,i)=objf_nodata
          rphotaeav(:,i)=objf_nodata
        case(2)
          call photos_smith(kmphotic,dz,parz,parzbot,kd, &
                            alpha(:,i),vp(:,i),photz(:,i),photav(:,i), &
                            rphotaeav(:,i))
        case(3)
          call photos_platt90(daylen,par0,kmphotic,dz,parfraczbot,kd, &
                              alpha(:,i),vp(:,i),photav(:,i))
          photz(:,i)=objf_nodata
          rphotaeav(:,i)=objf_nodata
        end select

        do k=kmphotic+1,km
          photz(k,i)=0.0
          photav(k,i)=0.0
          rphotaeav(k,i)=0.0
        enddo

      enddo

      end subroutine

!=======================================================================

      subroutine a93optics(km,rtpig,kd)

      implicit none

      integer km
      real(kind_r) rtpig(:)
      real(kind_r) kd(:)

!-----------------------------------------------------------------------

      integer k

      do k=1,km

        kd(k) = kdb0(k) + kdb1(k)*rtpig(k) + kdb2(k)*(rtpig(k)**2) &
          + kdb3(k)*(rtpig(k)**3) + kdb4(k)*(rtpig(k)**4) &
          + kdb5(k)*(rtpig(k)**5)

      enddo

      end subroutine 

!=======================================================================
      
      subroutine og99patheffect(lat,declin,kmax,kd)
      
      implicit none
      
      real(kind_r) lat
      real(kind_r) declin
      integer kmax
      real(kind_r) kd(:)

!-----------------------------------------------------------------------
      
      real(kind_r) costheta_a,sintheta_w,pathfact 
      integer k
     
!     Calculate cos of solar radiation incidence angle at noon
     
      costheta_a=max(0.0,sin(lat)*sin(declin)+cos(lat)*cos(declin))

!     Calculate sin of angle of refraction using Snell's law 
!     The refractive indices of water and air are approximately 1.333 and 1

      sintheta_w=sqrt(1.0_kind_r-costheta_a**2)/1.33_kind_r

!     Calculate correction factor for effective path length z/cos(theta_w)
!     and apply to attenuation coefficients

      pathfact=1.0_kind_r/sqrt(1.0_kind_r-sintheta_w**2)      
     
      do k=1,kmax
        kd(k)=kd(k)*pathfact
      enddo
      
      end subroutine

!=======================================================================

      subroutine a93spectralachl(km,dz,rtpig,ahash)

!     Calculate the spectrally everaged value of 'ahash' (a#) according 
!     to the approximation of Anderson 1993.
!     At any given wavelength a# is the dimensionless chlorophyll-specific
!     light absorption a* adjusted by a correction factor to allow for the 
!     conversion between downwelling and scalar irradiance. The correction
!     factor is the ratio of the downwelling attenuation coefficient and 
!     the absorption at the given wavelength.

      implicit none

      integer km 
      real(kind_r) dz(:) 
      real(kind_r) rtpig(:) 
      real(kind_r) ahash(:) 

!-----------------------------------------------------------------------

      real(kind_r) ahash0(0:km) 
      real(kind_r) dahash(km) 
      integer k

      real(kind_r) h(0:4)  ! Coeffs. in expression of surface chlorophyll-
                           !   specific absorption as a function of 
                           !   pigment conc.
      real(kind_r) g(10)   ! Coeffs. in expression for chlorophyll-
                           !  specific absorption as a function of 
                           !  depth and pigment concentration.

      data h / 3.6796E-1 , 1.7537E-1 , -6.5276E-2 , &
               1.3528E-2 , -1.1108E-3 /
      data g / 4.8014E-2 , 2.3779E-4 , -2.3074E-2 , 3.1095E-3 , -9.0545E-3 , &
               2.7974E-3 , 8.5217E-4 , -3.9804E-6 , 1.2398E-3 , -6.1991E-4 /

!     Calculate surface a# for each pigment concentration in the vertical 
!     profile. This is used to calculate offsets associated with pigment 
!     changes at depth level boundaries.

      do k=1,km
        ahash0(k) = h(0) + h(1)*rtpig(k) + h(2)*(rtpig(k)**2) + &
                    h(3)*(rtpig(k)**3) + h(4)*(rtpig(k)**4) 
      enddo

      ahash0(0) = ahash0(1)

!     The vertical gradient of a# is given by
!     d(a#)/dz = g1 + g2*c + g3*v +g4*c*v + g5*c^2 + g6*v^2
!              + g7*c^3 + g8*v^3 + g9*c^2*v + g10*c*v^2
!     where c=sqrt(pigment)
!           v=ln(z+1), z is depth in m
!           g1..g10 are fitted coefficients
!
!     The definite integral of this gradient gives the change in a# between
!     any two depths in a water column with constant pigment concentration.
!     It is used here to calculate level integrals.     

      do k=1,km
  
        dahash(k) = &
          dz(k)*(g(1)+g(2)*rtpig(k)+g(5)*(rtpig(k)**2)+g(7)*(rtpig(k)**3)) + &
          co_dahash(1,k)*(g(3)+g(4)*rtpig(k)+g(9)*(rtpig(k)**2)) + &
          co_dahash(2,k)*(g(6)+g(10)*rtpig(k)) + &
          co_dahash(3,k)*g(8)

      enddo

!     a#(k) is the level-mid-point value, approximated by adding half of
!     the level integrals of the a# gradient for levels k-1 and k to the
!     approximation a#(k-1) (or, for level 1,  adding the integral for level
!     k only to a#(0)).
!     Also add offsets to allow for pigment changes at level boundaries.

      ahash(1)=ahash0(0)+0.5_kind_r*dahash(1)

      do k=2,km
        ahash(k)=ahash(k-1)+0.5_kind_r*(dahash(k-1)+dahash(k)) + &
                 ahash0(k)-ahash0(k-1)
      enddo

      end subroutine

!=======================================================================

      subroutine photos_og99ep(daylen,kmax,dz,parzbot,kd,alpha,vp,avphot)

      implicit none

      real(kind_r) daylen 
      integer kmax
      real(kind_r) dz(:) 
      real(kind_r) parzbot(0:) 
      real(kind_r) kd(:) 
      real(kind_r) alpha(:)
      real(kind_r) vp(:) 
      real(kind_r) avphot(:) 

!-----------------------------------------------------------------------

      real(kind_r) tau,tausq
      real(kind_r) parbot,dzlight
      real(kind_r) x,betaztop,betazbot
      integer k

      tau=0.5_kind_r*daylen
      tausq=tau*tau

      do k=1,kmax

        if (vp(k)<=0.0 .or. alpha(k)<=0.0) then

          avphot(k)=0.0

        else

!         Calculate level mean photsynthesis
!         If part of level is dark, we integrate over light part only and
!         adjust to get level mean (Analytical integral fails in the dark)

          if (daylen<=0.0 .or. parzbot(k-1)<par_threshold) then

            avphot(k) = 0.0

          else

            if (parzbot(k)<par_threshold) then
              parbot=par_threshold
              dzlight=(log(parzbot(k-1))-log(par_threshold))/kd(k)
            else
              parbot=parzbot(k)
              dzlight=dz(k)
            endif

            if (dzlight<=0.0) then

              avphot(k)=0.0

            else

              x=vp(k)*tausq/alpha(k)
              betaztop=x/parzbot(k-1)
              betazbot=x/parbot
              avphot(k) = (2.0_kind_r*vp(k)/(kd(k)*dzlight)) * &
                          ( func_ep85(betazbot,tau) &
                            - func_ep85(betaztop,tau) &
                            - betazbot + betaztop )

              if (dzlight<dz(k)) then
                avphot(k)=dzlight*avphot(k)/dz(k)
              endif

            endif

          endif

        endif

      enddo

      end subroutine

!=======================================================================

      real(kind_r) function func_ep85(y,t)

      implicit none

      real(kind_r) y 
      real(kind_r) t

!-----------------------------------------------------------------------

      real(kind_r) x

      x=sqrt(y**2+t**2)
      func_ep85=x-t*log((t+x)/y)

      end function 

!=======================================================================

      subroutine photos_smith(kmax,dz,parz,parzbot,kd,alpha,vp, &
                              phot,avphot,avrphotae)

      implicit none

      integer kmax
      real(kind_r) dz(:)
      real(kind_r) parz(:)
      real(kind_r) kd(:)
      real(kind_r) parzbot(0:)
      real(kind_r) alpha(:)
      real(kind_r) vp(:)
      real(kind_r) phot(:)
      real(kind_r) avphot(:)
      real(kind_r) avrphotae(:)

!-----------------------------------------------------------------------

      real(kind_r) alphasq,vpsq
      real(kind_r) y1,y2,y1sq,y2sq
      real(kind_r) parbot,dzlight
      integer k

      do k=1,kmax

        if (vp(k)<=0.0) then

          phot(k)=0.0
          avphot(k)=0.0
          avrphotae(k)=0.0

        else if (alpha(k)<=0.0 .or. parzbot(0)<=0.0) then

          phot(k)=0.0
          avphot(k)=0.0
          avrphotae(k)=1.0_kind_r

        else

          vpsq=vp(k)*vp(k)
          alphasq=alpha(k)*alpha(k)

!         Calculate level mid-point photsynthesis

          phot(k)=vp(k)*alpha(k)*parz(k)/sqrt(vpsq+alphasq*parz(k)*parz(k))

          if (parzbot(k-1) < par_threshold) then

            avphot(k)=0.0
            avrphotae(k)=1.0_kind_r

          else

!           Calculate level mean photsynthesis

            y1=alpha(k)*parzbot(k-1)
            y1sq=y1*y1
            y2=alpha(k)*parzbot(k)
            y2sq=y2*y2
            avphot(k)=(vp(k)/(kd(k)*dz(k)))* &
              log((y1+sqrt(y1sq+vpsq))/(y2+sqrt(y2sq+vpsq)))

!           Calculate level mean photosynthesis:(alpha*par) ratio
!           If part of level is dark, we integrate over light part only and 
!           adjust to get level mean (Analytical integral fails in the dark)
!           Ratio tends to 1 at low light so taken to be 1 below threshold PAR

            if (parzbot(k)<par_threshold) then
              y2=alpha(k)*par_threshold
              y2sq=y2*y2
              dzlight=(log(parzbot(k-1))-log(par_threshold))/kd(k)
            else
              dzlight=dz(k)
            endif

            if (dzlight<=0.0) then
              avrphotae(k)=1.0_kind_r
            else
              avrphotae(k)=log(((vp(k)/y2)+sqrt((vpsq/y2sq)+1.0_kind_r))/ &
                               ((vp(k)/y1)+sqrt((vpsq/y1sq)+1.0_kind_r)))/ &
                           (kd(k)*dzlight)
              if (dzlight<dz(k)) then
                avrphotae(k)=(dzlight*avrphotae(k)+dz(k)-dzlight)/dz(k)
              endif 
            endif

          endif

        endif

      enddo

      end subroutine

!=======================================================================

      subroutine photos_platt90(daylen,par0,km,dz,parfraczbot,kd, &
                                alpha,vp,photav)

      implicit none

      real(kind_r) daylen
      real(kind_r) par0
      integer km
      real(kind_r) dz(:)
      real(kind_r) parfraczbot(0:)
      real(kind_r) kd(:) 
      real(kind_r) alpha(:)
      real(kind_r) vp(:)
      real(kind_r) photav(:)

!-----------------------------------------------------------------------

      real(kind_r) noonpar0
      real(kind_r) pi
      real(kind_r) istar_top 
      real(kind_r) istar_bot 
      integer k

!     Calculate noon PAR from daily mean, assuming a sinusoidal 
!     variation during the day

      pi=4.0_kind_r*atan(1.0_kind_r)
      if (daylen > 0.0 .and. par0 > 0.0) then
        noonpar0=par0*pi/(2.0_kind_r*daylen)
      else
        noonpar0=0.0
      endif

!     Calculate mean photosyntheis for each level from difference between 
!     depth-integrated photosynthesis below top of level and 
!     depth-integrated photosynthesis below bottom of level 

      do k=1,km
        if (vp(k) > 0.0) then
          istar_top=noonpar0*parfraczbot(k-1)*alpha(k)/vp(k)
          istar_bot=noonpar0*parfraczbot(k)*alpha(k)/vp(k)
          photav(k)= &
            (daylen*vp(k)/(pi*kd(k)*dz(k))) * &
            (pfunc(istar_top)-pfunc(istar_bot))
        else
          photav(k)=0.0
        endif
      enddo

      end subroutine

!=======================================================================

      real(kind_r) function pfunc(istar)

!     Returns Platt's polynomial approximation at 'istar' values below 16.8.
!     This breaks down at high values of 'istar' and is replaced by a 
!     rational function approximation above 16.8. 

      implicit none

      real(kind_r) istar

!-----------------------------------------------------------------------

      real(kind_r) omega(5)  ! Coefficients in polynomial expansion of 
                             ! depth-integrated daily photosynthesis 
                             ! (Platt et al, 1990)
      real(kind_r) psi(3)    ! Coefficients in rational function expansion 
                             ! of depth-integrated daily photosynthesis 
                             ! from work by Geoff Evans and Tom Anderson, 
                             ! subsequently used by Ian Totterdell in the 
                             ! Met Office HadOCC model 
                             ! (Totterdell, pers. comm., 2004) 

      data omega / 1.9004 , -2.8333E-1 , 2.8050E-2 , -1.4729E-3 , 3.0841E-5 /
      data psi / 1.62461 , 4.5412e-3 , 1.3140e-1 /

      if (istar < 16.8) then
        pfunc = omega(1)*istar + omega(2)*istar**2 + omega(3)*istar**3 + &
                omega(4)*istar**4 + omega(5)*istar**5    
      else
        pfunc = (psi(1)*istar + psi(2)*istar**2)/(1.0_kind_r + psi(3)*istar)
      endif

      end function

!=======================================================================

      end module photos 

