!**************************************************************************!
! SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1 !
!                                                                          !
! Includes modification to 2013 MarMOT 1.1 code (MarMOT 1.1.0) as noted    !
!**************************************************************************!
!**************************************************************************!
!                                                                          ! 
! SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013                     ! 
!                                                                          !
! Copyright Natural Environment Research Council, UK                       !
!                                                                          !
! Contains code derived from NEMO 3.4 (Copyright NEMO Consortium)          !
!                                                                          !
! Author: John Hemmings, National Oceanography Centre                      !
! e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013)                         !
! Alternative e-mail for author: marmot@wessexenv.mail1.co.uk              !
!                                                                          !
! This software is governed by the CeCILL  license under French law and    !
! abiding by the rules of distribution of free software.  You can  use,    ! 
! modify and/ or redistribute the software under the terms of the CeCILL   !
! license as circulated by CEA, CNRS and INRIA at the following URL        !
! "http://www.cecill.info".                                                ! 
!                                                                          !
! As a counterpart to the access to the source code and  rights to copy,   !
! modify and redistribute granted by the license, users are provided only  !
! with a limited warranty  and the software's author,  the holder of the   !
! economic rights,  and the successive licensors  have only  limited       !
! liability.                                                               ! 
!                                                                          !
! In this respect, the user's attention is drawn to the risks associated   !
! with loading,  using,  modifying and/or developing or reproducing the    !
! software by the user in light of its specific status of free software,   !
! that may mean  that it is complicated to manipulate,  and  that  also    !
! therefore means  that it is reserved for developers  and  experienced    !
! professionals having in-depth computer knowledge. Users are therefore    !
! encouraged to load and test the software's suitability as regards their  !
! requirements in conditions enabling the security of their systems and/or ! 
! data to be ensured and,  more generally, to use and operate it in the    ! 
! same conditions as regards security.                                     ! 
!                                                                          ! 
! The fact that you are presently reading this means that you have had     !
! knowledge of the CeCILL license and that you accept its terms.           !
!                                                                          !
!**************************************************************************!
!=======================================================================
! MODULE tradvect
! John Hemmings - 13 Nov 2013
!
! MarMOT advection subroutines: 
!
!  - advection - calculate vertical advection tendencies
!
!     - hinflux - tendency due to horizontal inflow implied by continuity
!     - advect_upstream - Upstream Scheme (Donor Cell Scheme)  
!     - advect_muscl - Monotone Upstream Scheme for Conservative Laws 
!                      (Van Leer 1977; Hourdin & Armengaud, 1999)
!
! Implementation of the MUSCL scheme is based on NEMO 3.4 code (Madec, 
! 2008). MUSCL was implemented, in the earlier OPA model, and evaluated 
! for biogeochemical modelling by Levy et al 2001, GRL 28, 3725-3728. 
!
! References:
! Hourdin F. and A. Armengaud, 1999. The Use of Finite-Volume Methods for 
! Atmospheric Advection of Trace Species. Part I: Test of Various 
! Formulations in a General Circulation Model, Mon. Weather Rev., 127, 
! 822-837. 
! Lévy, M., Estublier, A. & Madec, G., 2001. Choice of an advection scheme
! for biogeochemical models. Geophys. Res. Letters 28, 3725-3728.
! Madec, G., 2008. "NEMO ocean engine". Note du Pole de modélisation de 
! l'Institut Pierre-Simon Laplace (IPSL), France, No 27, ISSN No 1288-1619.
! (Version 3.4: Gurvan Madec & the NEMO team, January 2012)
! Van Leer, B., 1977. Towards the ultimate conservative difference scheme
! IV: a new approach to numerical convection, J. Comput. Phys. 23, 276-299.
!
! MODIFIED:
!
! John Hemmings - 02 Nov 2014 
! Fixed type mismatches between kind_r_ext and kind_r  
!=======================================================================

  module tradvect

  use modconfig, only: kind_r_ext,kind_r

  implicit none

  contains

!=======================================================================

  subroutine advection(scheme,km,dz,dt,w_passive,w_active,tracer,ddt,ddth)

  use trlink, only: ntracer,itracer

  implicit none

! Input arguments

  integer scheme                 ! Advection scheme id
  integer km                     ! Maximum depth level
  real(kind_r) dz(:)             ! Level height (m)
  real(kind_r) dt                ! Time step (d)
  real(kind_r_ext) w_passive(:)  ! Vertical advection (m/d, up)
  real(kind_r) w_active(:,:)     ! Added tracer-specific vel. (m/d,up)
  real(kind_r_ext) tracer(:,:)   ! Tracer concentrations 

! In/out argument

  real(kind_r_ext) ddt(:,:)      ! Tendencies for update (/d)

! Output argument

  real(kind_r) ddth(:,:)         ! Horizontal influxes (units/d)

!-----------------------------------------------------------------------

  real(kind_r) w(km)
  integer i,j,k

  do i=1,ntracer
    j=itracer(i)
    if (j/=0) then

      call hinflux(km,dz,w_passive,tracer(:,j),ddth(:,j))

      w(1:km)=w_passive(1:km)+w_active(1:km,j)

      select case(scheme)
      case(1)
        call advect_upstream(km,dz,w,tracer(:,j),ddth(:,j),ddt(:,j))
      case(2)
        call advect_muscl(km,dz,dt,w,tracer(:,j),ddth(:,j),ddt(:,j))
      end select

    endif
  enddo

  end subroutine

!=======================================================================

  subroutine hinflux(km,dz,w,tracer,ddt)

  implicit none

! Input arguments

  integer km                    ! Maximum depth level
  real(kind_r) dz(:)            ! Level height (m)
  real(kind_r_ext) w(:)
  real(kind_r_ext) tracer(:)

! Output argument

  real(kind_r) ddt(:)

!-----------------------------------------------------------------------

  integer k

! Tendency due to horizontal flux divergence inferred by continuity
! in the flow (horizontal tracer gradient assumed to be zero).

  ddt(1)=-w(1)*tracer(1)/dz(1)
  do k=2,km-1
    ddt(k)=(w(k-1)-w(k))*tracer(k)/dz(k)
  enddo
  ddt(km)=w(k-1)*tracer(k)/dz(k)

  end subroutine

!=======================================================================

  subroutine advect_upstream(km,dz,w,tracer,ddth,ddt)

  implicit none

! Input arguments

  integer km                     ! Maximum depth level
  real(kind_r) dz(:)             ! Level height (m)
  real(kind_r) w(:)
  real(kind_r_ext) tracer(:)
  real(kind_r) ddth(:)

! In/out argument

  real(kind_r_ext) ddt(:)

!-----------------------------------------------------------------------

  real(kind_r) up,down 
  real(kind_r) fluxzbot(0:km)
  integer k

  fluxzbot(0)=0.0  
  do k=1,km-1
    up=0.5_kind_r+sign(0.5_kind_r,w(k))
    down=0.5_kind_r-sign(0.5_kind_r,w(k))
    fluxzbot(k)=w(k)*(down*tracer(k)+up*tracer(k+1))
  enddo
  fluxzbot(km)=0.0  

! Add tendencies from vertical and horizontal flux divergence

  do k=1,km
    ddt(k)=ddt(k)+(fluxzbot(k)-fluxzbot(k-1))/dz(k)+ddth(k)
  enddo

  end subroutine

!=======================================================================

  subroutine advect_muscl(km,dz,dt,w,tracer,ddth,ddt)

  implicit none

! Input arguments

  integer km                     ! Maximum depth level
  real(kind_r) dz(:)             ! Level height (m)
  real(kind_r) dt
  real(kind_r_ext) tracer(:)
  real(kind_r) w(:)
  real(kind_r) ddth(:)

! In/out argument

  real(kind_r_ext) ddt(:)

!-----------------------------------------------------------------------

  real(kind_r) x(0:km)
  real(kind_r) slope(km)
  real(kind_r) a,b,zw,trupw,trdownw
  integer kmm1,k

  kmm1=km-1

! Slope (up column) at velocity points (bottom of level)

  do k=1,kmm1
    x(k)=tracer(k)-tracer(k+1)
  enddo

! Slope at tracer points (middle of level) with limitation to
! ensure positivity

  slope(1)=0.0
  do k=2,kmm1
    slope(k)=(x(k-1)+x(k))*(0.25_kind_r+sign(0.25_kind_r,x(k-1)*x(k)))
    slope(k)=sign(1.0_kind_r,slope(k))* &
             min(abs(slope(k)),2.0_kind_r*abs(x(k)),2.0_kind_r*abs(x(k-1)))
  enddo
  slope(km)=0.0

! Advective flux upward at velocity points. Tracer value at the velocity 
! point is taken to be the time-step average at that point ('trupw' for 
! upwelling or 'trdownw' for downwelling).  

  x(0)=0.0
  do k=1,kmm1
    a=sign(0.5_kind_r,w(k))  
    b=0.5_kind_r+a
    zw=a-0.5_kind_r*w(k)*dt/dz(k) 
    trupw=tracer(k+1)+zw*slope(k+1)  
    trdownw=tracer(k)+zw*slope(k)
    x(k)=w(k)*(b*trupw+(1.0_kind_r-b)*trdownw) 
  enddo
  x(km)=0.0

! Add tendencies from vertical and horizontal flux divergence

  do k=1,km
    ddt(k)=ddt(k)+(x(k)-x(k-1))/dz(k)+ddth(k)
  enddo

  end subroutine

!=======================================================================

  end module tradvect
