!**************************************************************************!
! SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1 !
!                                                                          !
! Includes modifications to 2013 MarMOT 1.1 code (MarMOT 1.1.0) as listed  !
!  - Copyright 2015 John C. P. Hemmings                                    !
!**************************************************************************!
!**************************************************************************!
!                                                                          ! 
! SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013                     ! 
!                                                                          !
! Copyright Natural Environment Research Council, UK                       !
!                                                                          !
! Author: John Hemmings, National Oceanography Centre                      !
! e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013)                         !
! Alternative e-mail for author: marmot@wessexenv.mail1.co.uk              !
!                                                                          !
! This software is governed by the CeCILL  license under French law and    !
! abiding by the rules of distribution of free software.  You can  use,    ! 
! modify and/ or redistribute the software under the terms of the CeCILL   !
! license as circulated by CEA, CNRS and INRIA at the following URL        !
! "http://www.cecill.info".                                                ! 
!                                                                          !
! As a counterpart to the access to the source code and  rights to copy,   !
! modify and redistribute granted by the license, users are provided only  !
! with a limited warranty  and the software's author,  the holder of the   !
! economic rights,  and the successive licensors  have only  limited       !
! liability.                                                               ! 
!                                                                          !
! In this respect, the user's attention is drawn to the risks associated   !
! with loading,  using,  modifying and/or developing or reproducing the    !
! software by the user in light of its specific status of free software,   !
! that may mean  that it is complicated to manipulate,  and  that  also    !
! therefore means  that it is reserved for developers  and  experienced    !
! professionals having in-depth computer knowledge. Users are therefore    !
! encouraged to load and test the software's suitability as regards their  !
! requirements in conditions enabling the security of their systems and/or ! 
! data to be ensured and,  more generally, to use and operate it in the    ! 
! same conditions as regards security.                                     ! 
!                                                                          ! 
! The fact that you are presently reading this means that you have had     !
! knowledge of the CeCILL license and that you accept its terms.           !
!                                                                          !
!**************************************************************************!
!=======================================================================
!
!  MODULE trlink 
!  John Hemmings - 18 Apr 2013 
!
!  MarMOT tracer linking subroutine: linkedtrend
!
!  Links derived tracer tendencies due to perturbation and advection
!  to the equivalent primary tracer tendencies.
!  The tendency ratio is that of the tracer ratio constrained to an 
!  acceptable range defined by the ecosystem model.
!  This constraint avoids the use of unrealistic tendency ratios that
!  can otherwise occur when tracer values are close to zero. 
!  If the base tracer concentration is zero or either tracer is negative
!  then the tracer ratio is invalid and a model-specified default is 
!  used for the tendency ratio. 
!
!  MODIFIED:
!
!  John Hemmings - 25 Nov 2014
!  - Fixed type mismatch between kind_r_ext and kind_r 
!  - Introduced 'kind_i_ext' for integer type used in marmot.c interface 
!  - Introduced use of ISO C Binding features for portability
!  (suppressed if 'no_iso_c_binding' is defined)
!
!=======================================================================

  module trlink

#ifndef no_iso_c_binding
  use iso_c_binding
#endif

  use modconfig, only: kind_i_ext,kind_r_ext,kind_r,nt

  implicit none

  integer,parameter :: ntpri=13     ! Number of MarMOT primary tracers

  ! Copy of parameter value for marmot.c

#ifdef no_iso_c_binding
  integer(kind_i_ext) :: modelntpri=ntpri 
#else
  integer(kind_i_ext),bind(C,name="FORTRAN_modelntpri") :: modelntpri=ntpri
#endif

! Model-specific configuration data
!
! Note: the ecosystem model specification in module 'modconfig' determines  
! the subsets of MarMOT tracers activated for use with the selected
! model. Index vector 'itracer' associated with this activated subset is used   
! to access the MarMOT tracers. Tracer indices may be set to zero by the model 
! to suppress tracers in the activated subset that are not used in the current 
! model configuration. 

  integer ntracer      ! Number of tracers activated (primary+derived)
  integer ntracer_in   ! Number of primary tracers activated 
  integer itracer(nt)  ! MarMOT tracer indices for activated tracers

  integer ntracer_linked    ! Number of derived tracers linked to primaries
  integer ibasetracer(nt-ntpri)
                            ! Model indices of primaries they are linked to
  real(kind_r) linkratio_def(nt-ntpri) 
                            ! Default tendency ratio for derived tracers
  real(kind_r) linkratio_min(nt-ntpri) ! Min. tendency ratio for derived tracers
  real(kind_r) linkratio_max(nt-ntpri) ! Max. tendency ratio for derived tracers

  contains

!=======================================================================

  subroutine linkedtrend(kmin,kmax,tracer,ddt)

  implicit none

! Input arguments

  integer kmin
  integer kmax
  real(kind_r_ext) tracer(:,:)

! In/out argument

  real(kind_r) ddt(:,:)

!-----------------------------------------------------------------------

  real(kind_r) linkratio
  integer i,j,jb,k

  do i=1,ntracer_linked
    j=itracer(ntracer_in+i)
    jb=itracer(ibasetracer(i))
    if (j/=0 .and. jb/=0) then
      do k=kmin,kmax
        if (tracer(k,j)<0.0 .or. tracer(k,jb)<=0.0) then
          linkratio=linkratio_def(i)
        else
          linkratio=tracer(k,j)/tracer(k,jb)
          linkratio=min(linkratio_max(i),max(linkratio_min(i),linkratio))
        endif
        ddt(k,j)=ddt(k,jb)*linkratio
      enddo
    endif
  enddo

  end subroutine linkedtrend
  
!=======================================================================

  end module trlink
